=== added file 'BRANCH.TODO'
--- BRANCH.TODO	1970-01-01 00:00:00 +0000
+++ BRANCH.TODO	2006-01-03 05:21:37 +0000
@@ -0,0 +1,3 @@
+# This file is for listing TODOs for branches that are being worked on.
+# It should ALWAYS be empty in the mainline or in integration branches.
+# 

=== modified file 'Makefile'
--- Makefile	2005-12-12 23:14:43 +0000
+++ Makefile	2006-01-03 05:21:37 +0000
@@ -10,7 +10,8 @@
 
 clean: 
 	./setup.py clean
-	find . -name "*.pyc" | xargs rm
+	-find . -name "*.pyc" | xargs rm
+	-find . -name "*.pyo" | xargs rm
 	rm -rf test????.tmp
 
 .PHONY: all

=== modified file 'NEWS'
--- NEWS	2005-12-16 01:47:35 +0000
+++ NEWS	2006-01-02 22:54:45 +0000
@@ -35,6 +35,10 @@
       or
       sftp://user:pass@host:port//absolute/path
 
+    * Permissions on files underneath .bzr/ are inherited from the .bzr 
+      directory. So for a shared repository, simply doing 'chmod -R g+w .bzr/'
+      will mean that future file will be created with group write permissions.
+
     * configure.in and config.guess are no longer in the builtin default 
       ignore list.
 

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-12-16 01:47:35 +0000
+++ bzrlib/__init__.py	2006-01-03 15:33:37 +0000
@@ -95,7 +95,10 @@
 
 
 def get_bzr_revision():
-    """If bzr is run from a branch, return (revno,revid) or None"""
+    """If bzr is run from a branch, return (revno,revid) or None
+
+    deprecated in bzr 0.7
+    """
     import bzrlib.errors
     from bzrlib.branch import Branch
     

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-12-15 08:32:03 +0000
+++ bzrlib/add.py	2005-12-18 04:08:46 +0000
@@ -151,7 +151,7 @@
 
         if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
-                subp = os.path.join(rf, subf)
+                subp = bzrlib.osutils.pathjoin(rf, subf)
                 if subf == bzrlib.BZRDIR:
                     mutter("skip control directory %r", subp)
                 else:

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2005-10-22 23:23:56 +0000
+++ bzrlib/atomicfile.py	2005-12-18 04:08:46 +0000
@@ -17,7 +17,7 @@
 
 
 from warnings import warn
-from osutils import rename
+from bzrlib.osutils import rename
 import errno
 
 class AtomicFile(object):
@@ -31,7 +31,7 @@
     An encoding can be specified; otherwise the default is ascii.
     """
 
-    def __init__(self, filename, mode='wb', encoding=None):
+    def __init__(self, filename, mode='wb', encoding=None, new_mode=None):
         if mode != 'wb' and mode != 'wt':
             raise ValueError("invalid AtomicFile mode %r" % mode)
 
@@ -48,6 +48,7 @@
 
         self.write = self.f.write
         self.closed = False
+        self._new_mode = new_mode
 
 
     def __repr__(self):
@@ -67,11 +68,14 @@
         self.f = None
         
         try:
-            stat = os.lstat(self.realfilename)
-            os.chmod(self.tmpfilename, stat.st_mode)
+            if self._new_mode is None:
+                self._new_mode = os.lstat(self.realfilename).st_mode
         except OSError, e:
             if e.errno != errno.ENOENT:
                 raise
+        else:
+            os.chmod(self.tmpfilename, self._new_mode)
+
         rename(self.tmpfilename, self.realfilename)
 
 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-12-31 15:19:21 +0000
+++ bzrlib/branch.py	2006-01-03 23:51:45 +0000
@@ -27,8 +27,8 @@
 import bzrlib.inventory as inventory
 from bzrlib.trace import mutter, note
 from bzrlib.osutils import (isdir, quotefn,
-                            rename, splitpath, sha_file, appendpath, 
-                            file_kind, abspath)
+                            rename, splitpath, sha_file,
+                            file_kind, abspath, normpath, pathjoin)
 import bzrlib.errors as errors
 from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
                            NoSuchRevision, HistoryMissing, NotBranchError,
@@ -133,8 +133,8 @@
         while True:
             try:
                 return BzrBranch(t), t.relpath(url)
-            except NotBranchError:
-                pass
+            except NotBranchError, e:
+                mutter('not a branch in: %r %s', t.base, e)
             new_t = t.clone('..')
             if new_t.base == t.base:
                 # reached the root, whatever that may be
@@ -518,6 +518,10 @@
     _lock = None
     _inventory_weave = None
     _master_branch = None
+    # If set to False (by a plugin, etc) BzrBranch will not set the
+    # mode on created files or directories
+    _set_file_mode = True
+    _set_dir_mode = True
     
     # Map some sort of prefix into a namespace
     # stuff like "revno:10", "revid:", etc.
@@ -568,25 +572,23 @@
         if init:
             self._make_control()
         self._check_format(relax_version_check)
+        self._find_modes()
 
         def get_store(name, compressed=True, prefixed=False):
-            # FIXME: This approach of assuming stores are all entirely compressed
-            # or entirely uncompressed is tidy, but breaks upgrade from 
-            # some existing branches where there's a mixture; we probably 
-            # still want the option to look for both.
             relpath = self._rel_controlfilename(unicode(name))
             store = TextStore(self._transport.clone(relpath),
+                              dir_mode=self._dir_mode,
+                              file_mode=self._file_mode,
                               prefixed=prefixed,
                               compressed=compressed)
-            #if self._transport.should_cache():
-            #    cache_path = os.path.join(self.cache_root, name)
-            #    os.mkdir(cache_path)
-            #    store = bzrlib.store.CachedStore(store, cache_path)
             return store
 
         def get_weave(name, prefixed=False):
             relpath = self._rel_controlfilename(unicode(name))
-            ws = WeaveStore(self._transport.clone(relpath), prefixed=prefixed)
+            ws = WeaveStore(self._transport.clone(relpath),
+                            prefixed=prefixed,
+                            dir_mode=self._dir_mode,
+                            file_mode=self._file_mode)
             if self._transport.should_cache():
                 ws.enable_cache = True
             return ws
@@ -760,7 +762,25 @@
                     f = codecs.getwriter('utf-8')(f, errors='replace')
             path = self._rel_controlfilename(path)
             ctrl_files.append((path, f))
-        self._transport.put_multi(ctrl_files)
+        self._transport.put_multi(ctrl_files, mode=self._file_mode)
+
+    def _find_modes(self, path=None):
+        """Determine the appropriate modes for files and directories."""
+        try:
+            if path is None:
+                path = self._rel_controlfilename('')
+            st = self._transport.stat(path)
+        except errors.TransportNotPossible:
+            self._dir_mode = 0755
+            self._file_mode = 0644
+        else:
+            self._dir_mode = st.st_mode & 07777
+            # Remove the sticky and execute bits for files
+            self._file_mode = self._dir_mode & ~07111
+        if not self._set_dir_mode:
+            self._dir_mode = None
+        if not self._set_file_mode:
+            self._file_mode = None
 
     def _make_control(self):
         from bzrlib.inventory import Inventory
@@ -778,7 +798,12 @@
         bzrlib.weavefile.write_weave_v5(Weave(), sio)
         empty_weave = sio.getvalue()
 
-        dirs = [[], 'revision-store', 'weaves']
+        cfn = self._rel_controlfilename
+        # Since we don't have a .bzr directory, inherit the
+        # mode from the root directory
+        self._find_modes(u'.')
+
+        dirs = ['', 'revision-store', 'weaves']
         files = [('README', 
             "This is a Bazaar-NG control directory.\n"
             "Do not change any files in this directory.\n"),
@@ -791,8 +816,7 @@
             ('inventory.weave', empty_weave),
             ('ancestry.weave', empty_weave)
         ]
-        cfn = self._rel_controlfilename
-        self._transport.mkdir_multi([cfn(d) for d in dirs])
+        self._transport.mkdir_multi([cfn(d) for d in dirs], mode=self._dir_mode)
         self.put_controlfiles(files)
         mutter('created control directory in ' + self._transport.base)
 
@@ -1025,7 +1049,7 @@
             return EmptyTree()
         else:
             inv = self.get_revision_inventory(revision_id)
-            return RevisionTree(self.weave_store, inv, revision_id)
+            return RevisionTree(self, inv, revision_id)
 
     def basis_tree(self):
         """See Branch.basis_tree."""
@@ -1033,7 +1057,7 @@
             revision_id = self.revision_history()[-1]
             xml = self.working_tree().read_basis_inventory(revision_id)
             inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
-            return RevisionTree(self.weave_store, inv, revision_id)
+            return RevisionTree(self, inv, revision_id)
         except (IndexError, NoSuchFile, NoWorkingTree), e:
             return self.revision_tree(self.last_revision())
 
@@ -1283,11 +1307,11 @@
         ...   orig.base == clone.base
         ...
         False
-        >>> os.path.isfile(os.path.join(clone.base, "file1"))
+        >>> os.path.isfile(pathjoin(clone.base, "file1"))
         True
         """
         from shutil import copytree
-        from tempfile import mkdtemp
+        from bzrlib.osutils import mkdtemp
         base = mkdtemp()
         os.rmdir(base)
         copytree(self.base, base, symlinks=True)
@@ -1301,7 +1325,7 @@
 
 def is_control_file(filename):
     ## FIXME: better check
-    filename = os.path.normpath(filename)
+    filename = normpath(filename)
     while filename != '':
         head, tail = os.path.split(filename)
         ## mutter('check %r for control file' % ((head, tail), ))

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-12-31 00:23:03 +0000
+++ bzrlib/builtins.py	2006-01-03 23:51:45 +0000
@@ -1470,12 +1470,31 @@
             bzrlib.ui.ui_factory = save_ui
 
 
+def _get_bzr_branch():
+    """If bzr is run from a branch, return Branch or None"""
+    import bzrlib.errors
+    from bzrlib.branch import Branch
+    from bzrlib.osutils import abspath
+    from os.path import dirname
+    
+    try:
+        branch = Branch.open(dirname(abspath(dirname(__file__))))
+        return branch
+    except bzrlib.errors.BzrError:
+        return None
+    
+
 def show_version():
     print "bzr (bazaar-ng) %s" % bzrlib.__version__
     # is bzrlib itself in a branch?
-    bzrrev = bzrlib.get_bzr_revision()
-    if bzrrev:
-        print "  (bzr checkout, revision %d {%s})" % bzrrev
+    branch = _get_bzr_branch()
+    if branch:
+        rh = branch.revision_history()
+        revno = len(rh)
+        print "  bzr checkout, revision %d" % (revno,)
+        print "  nick: %s" % (branch.nick,)
+        if rh:
+            print "  revid: %s" % (rh[-1],)
     print bzrlib.__copyright__
     print "http://bazaar-ng.org/"
     print

=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-12-11 20:28:41 +0000
+++ bzrlib/changeset.py	2005-12-22 23:32:43 +0000
@@ -25,23 +25,26 @@
 import os.path
 import errno
 import stat
-from tempfile import mkdtemp
 from shutil import rmtree
 from itertools import izip
 
 from bzrlib.trace import mutter, warning
-from bzrlib.osutils import rename, sha_file
+from bzrlib.osutils import rename, sha_file, pathjoin, mkdtemp
 import bzrlib
 from bzrlib.errors import BzrCheckError
 
 __docformat__ = "restructuredtext"
 
+
 NULL_ID = "!NULL"
 
+
 class OldFailedTreeOp(Exception):
     def __init__(self):
         Exception.__init__(self, "bzr-tree-change contains files from a"
                            " previous failed merge operation.")
+
+
 def invert_dict(dict):
     newdict = {}
     for (key,value) in dict.iteritems():
@@ -56,13 +59,9 @@
         self.old_exec_flag = old_exec_flag
         self.new_exec_flag = new_exec_flag
 
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            from_exec_flag = self.old_exec_flag
-            to_exec_flag = self.new_exec_flag
-        else:
-            from_exec_flag = self.new_exec_flag
-            to_exec_flag = self.old_exec_flag
+    def apply(self, filename, conflict_handler):
+        from_exec_flag = self.old_exec_flag
+        to_exec_flag = self.new_exec_flag
         try:
             current_exec_flag = bool(os.stat(filename).st_mode & 0111)
         except OSError, e:
@@ -105,7 +104,7 @@
         return not (self == other)
 
 
-def dir_create(filename, conflict_handler, reverse):
+def dir_create(filename, conflict_handler, reverse=False):
     """Creates the directory, or deletes it if reverse is true.  Intended to be
     used with ReplaceContents.
 
@@ -150,7 +149,7 @@
     def __repr__(self):
         return "SymlinkCreate(%s)" % self.target
 
-    def __call__(self, filename, conflict_handler, reverse):
+    def __call__(self, filename, conflict_handler, reverse=False):
         """Creates or destroys the symlink.
 
         :param filename: The name of the symlink to create
@@ -179,6 +178,7 @@
     def __ne__(self, other):
         return not (self == other)
 
+
 class FileCreate(object):
     """Create or delete a file (for use with ReplaceContents)"""
     def __init__(self, contents):
@@ -203,7 +203,7 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def __call__(self, filename, conflict_handler, reverse):
+    def __call__(self, filename, conflict_handler, reverse=False):
         """Create or delete a file
 
         :param filename: The name of the file to create
@@ -235,7 +235,6 @@
                 if conflict_handler.missing_for_rm(filename, undo) == "skip":
                     return
 
-                    
 
 class TreeFileCreate(object):
     """Create or delete a file (for use with ReplaceContents)"""
@@ -269,7 +268,7 @@
         in_file = file(filename, "rb")
         return sha_file(in_file) == self.tree.get_file_sha1(self.file_id)
 
-    def __call__(self, filename, conflict_handler, reverse):
+    def __call__(self, filename, conflict_handler, reverse=False):
         """Create or delete a file
 
         :param filename: The name of the file to create
@@ -301,12 +300,6 @@
                 if conflict_handler.missing_for_rm(filename, undo) == "skip":
                     return
 
-                    
-
-def reversed(sequence):
-    max = len(sequence) - 1
-    for i in range(len(sequence)):
-        yield sequence[max - i]
 
 class ReplaceContents(object):
     """A contents-replacement framework.  It allows a file/directory/symlink to
@@ -344,20 +337,14 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         """Applies the FileReplacement to the specified filename
 
         :param filename: The name of the file to apply changes to
         :type filename: str
-        :param reverse: If true, apply the change in reverse
-        :type reverse: bool
         """
-        if not reverse:
-            undo = self.old_contents
-            perform = self.new_contents
-        else:
-            undo = self.new_contents
-            perform = self.old_contents
+        undo = self.old_contents
+        perform = self.new_contents
         mode = None
         if undo is not None:
             try:
@@ -371,7 +358,7 @@
                     return
             undo(filename, conflict_handler, reverse=True)
         if perform is not None:
-            perform(filename, conflict_handler, reverse=False)
+            perform(filename, conflict_handler)
             if mode is not None:
                 os.chmod(filename, mode)
 
@@ -381,35 +368,6 @@
     def is_deletion(self):
         return self.old_contents is not None and self.new_contents is None
 
-class ApplySequence(object):
-    def __init__(self, changes=None):
-        self.changes = []
-        if changes is not None:
-            self.changes.extend(changes)
-
-    def __eq__(self, other):
-        if not isinstance(other, ApplySequence):
-            return False
-        elif len(other.changes) != len(self.changes):
-            return False
-        else:
-            for i in range(len(self.changes)):
-                if self.changes[i] != other.changes[i]:
-                    return False
-            return True
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            iter = self.changes
-        else:
-            iter = reversed(self.changes)
-        for change in iter:
-            change.apply(filename, conflict_handler, reverse)
-
 
 class Diff3Merge(object):
     history_based = False
@@ -434,26 +392,22 @@
         return not (self == other)
 
     def dump_file(self, temp_dir, name, tree):
-        out_path = os.path.join(temp_dir, name)
+        out_path = pathjoin(temp_dir, name)
         out_file = file(out_path, "wb")
         in_file = tree.get_file(self.file_id)
         for line in in_file:
             out_file.write(line)
         return out_path
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         import bzrlib.patch
         temp_dir = mkdtemp(prefix="bzr-")
         try:
             new_file = filename+".new"
             base_file = self.dump_file(temp_dir, "base", self.base)
             other_file = self.dump_file(temp_dir, "other", self.other)
-            if not reverse:
-                base = base_file
-                other = other_file
-            else:
-                base = other_file
-                other = base_file
+            base = base_file
+            other = other_file
             status = bzrlib.patch.diff3(new_file, filename, base, other)
             if status == 0:
                 os.chmod(new_file, os.stat(filename).st_mode)
@@ -482,6 +436,7 @@
     """
     return ReplaceContents(None, dir_create)
 
+
 def DeleteDir():
     """Convenience function to delete a directory.
 
@@ -490,6 +445,7 @@
     """
     return ReplaceContents(dir_create, None)
 
+
 def CreateFile(contents):
     """Convenience fucntion to create a file.
     
@@ -500,6 +456,7 @@
     """
     return ReplaceContents(None, FileCreate(contents))
 
+
 def DeleteFile(contents):
     """Convenience fucntion to delete a file.
     
@@ -510,6 +467,7 @@
     """
     return ReplaceContents(FileCreate(contents), None)
 
+
 def ReplaceFileContents(old_tree, new_tree, file_id):
     """Convenience fucntion to replace the contents of a file.
     
@@ -523,6 +481,7 @@
     return ReplaceContents(TreeFileCreate(old_tree, file_id), 
                            TreeFileCreate(new_tree, file_id))
 
+
 def CreateSymlink(target):
     """Convenience fucntion to create a symlink.
     
@@ -533,6 +492,7 @@
     """
     return ReplaceContents(None, SymlinkCreate(target))
 
+
 def DeleteSymlink(target):
     """Convenience fucntion to delete a symlink.
     
@@ -543,6 +503,7 @@
     """
     return ReplaceContents(SymlinkCreate(target), None)
 
+
 def ChangeTarget(old_target, new_target):
     """Convenience fucntion to change the target of a symlink.
     
@@ -586,6 +547,7 @@
         msg = 'Child of !NULL is named "%s", not "./.".' % name
         InvalidEntry.__init__(self, entry, msg)
 
+
 class NullIDAssigned(InvalidEntry):
     """The id !NULL was assigned to a real entry"""
     def __init__(self, entry):
@@ -597,6 +559,7 @@
         msg = '"!NULL" id assigned to a file "%s".' % entry.path
         InvalidEntry.__init__(self, entry, msg)
 
+
 class ParentIDIsSelf(InvalidEntry):
     """An entry is marked as its own parent"""
     def __init__(self, entry):
@@ -609,6 +572,7 @@
             (entry.path, entry.id)
         InvalidEntry.__init__(self, entry, msg)
 
+
 class ChangesetEntry(object):
     """An entry the changeset"""
     def __init__(self, id, parent, path):
@@ -644,7 +608,7 @@
         return os.path.dirname(self.path)
 
     def __set_dir(self, dir):
-        self.path = os.path.join(dir, os.path.basename(self.path))
+        self.path = pathjoin(dir, os.path.basename(self.path))
 
     dir = property(__get_dir, __set_dir)
     
@@ -654,7 +618,7 @@
         return os.path.basename(self.path)
 
     def __set_name(self, name):
-        self.path = os.path.join(os.path.dirname(self.path), name)
+        self.path = pathjoin(os.path.dirname(self.path), name)
 
     name = property(__get_name, __set_name)
 
@@ -664,7 +628,7 @@
         return os.path.dirname(self.new_path)
 
     def __set_new_dir(self, dir):
-        self.new_path = os.path.join(dir, os.path.basename(self.new_path))
+        self.new_path = pathjoin(dir, os.path.basename(self.new_path))
 
     new_dir = property(__get_new_dir, __set_new_dir)
 
@@ -674,7 +638,7 @@
         return os.path.basename(self.new_path)
 
     def __set_new_name(self, name):
-        self.new_path = os.path.join(os.path.dirname(self.new_path), name)
+        self.new_path = pathjoin(os.path.dirname(self.new_path), name)
 
     new_name = property(__get_new_name, __set_new_name)
 
@@ -686,7 +650,7 @@
 
         return (self.parent != self.new_parent or self.name != self.new_name)
 
-    def is_deletion(self, reverse):
+    def is_deletion(self, reverse=False):
         """Return true if applying the entry would delete a file/directory.
 
         :param reverse: if true, the changeset is being applied in reverse
@@ -694,7 +658,7 @@
         """
         return self.is_creation(not reverse)
 
-    def is_creation(self, reverse):
+    def is_creation(self, reverse=False):
         """Return true if applying the entry would create a file/directory.
 
         :param reverse: if true, the changeset is being applied in reverse
@@ -713,7 +677,7 @@
 
         :rtype: bool
         """
-        return self.is_creation(False) or self.is_deletion(False)
+        return self.is_creation() or self.is_deletion()
 
     def get_cset_path(self, mod=False):
         """Determine the path of the entry according to the changeset.
@@ -739,15 +703,11 @@
                 return None
             return self.path
 
-    def summarize_name(self, reverse=False):
+    def summarize_name(self):
         """Produce a one-line summary of the filename.  Indicates renames as
         old => new, indicates creation as None => new, indicates deletion as
         old => None.
 
-        :param changeset: The changeset to get paths from
-        :type changeset: `Changeset`
-        :param reverse: If true, reverse the names in the output
-        :type reverse: bool
         :rtype: str
         """
         orig_path = self.get_cset_path(False)
@@ -759,36 +719,23 @@
         if orig_path == mod_path:
             return orig_path
         else:
-            if not reverse:
-                return "%s => %s" % (orig_path, mod_path)
-            else:
-                return "%s => %s" % (mod_path, orig_path)
-
-
-    def get_new_path(self, id_map, changeset, reverse=False):
+            return "%s => %s" % (orig_path, mod_path)
+
+    def get_new_path(self, id_map, changeset):
         """Determine the full pathname to rename to
 
         :param id_map: The map of ids to filenames for the tree
         :type id_map: Dictionary
         :param changeset: The changeset to get data from
         :type changeset: `Changeset`
-        :param reverse: If true, we're applying the changeset in reverse
-        :type reverse: bool
         :rtype: str
         """
         mutter("Finding new path for %s", self.summarize_name())
-        if reverse:
-            parent = self.parent
-            to_dir = self.dir
-            from_dir = self.new_dir
-            to_name = self.name
-            from_name = self.new_name
-        else:
-            parent = self.new_parent
-            to_dir = self.new_dir
-            from_dir = self.dir
-            to_name = self.new_name
-            from_name = self.name
+        parent = self.new_parent
+        to_dir = self.new_dir
+        from_dir = self.dir
+        to_name = self.new_name
+        from_name = self.name
 
         if to_name is None:
             return None
@@ -803,13 +750,13 @@
             dir = os.path.dirname(id_map[self.id])
         else:
             mutter("path, new_path: %r %r", self.path, self.new_path)
-            dir = parent_entry.get_new_path(id_map, changeset, reverse)
+            dir = parent_entry.get_new_path(id_map, changeset)
         if from_name == to_name:
             name = os.path.basename(id_map[self.id])
         else:
             name = to_name
             assert(from_name is None or from_name == os.path.basename(id_map[self.id]))
-        return os.path.join(dir, name)
+        return pathjoin(dir, name)
 
     def is_boring(self):
         """Determines whether the entry does nothing
@@ -828,20 +775,19 @@
         else:
             return True
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         """Applies the file content and/or metadata changes.
 
         :param filename: the filename of the entry
         :type filename: str
-        :param reverse: If true, apply the changes in reverse
-        :type reverse: bool
         """
-        if self.is_deletion(reverse) and self.metadata_change is not None:
-            self.metadata_change.apply(filename, conflict_handler, reverse)
+        if self.is_deletion() and self.metadata_change is not None:
+            self.metadata_change.apply(filename, conflict_handler)
         if self.contents_change is not None:
-            self.contents_change.apply(filename, conflict_handler, reverse)
-        if not self.is_deletion(reverse) and self.metadata_change is not None:
-            self.metadata_change.apply(filename, conflict_handler, reverse)
+            self.contents_change.apply(filename, conflict_handler)
+        if not self.is_deletion() and self.metadata_change is not None:
+            self.metadata_change.apply(filename, conflict_handler)
+
 
 class IDPresent(Exception):
     def __init__(self, id):
@@ -850,6 +796,7 @@
         Exception.__init__(self, msg)
         self.id = id
 
+
 class Changeset(object):
     """A set of changes to apply"""
     def __init__(self):
@@ -861,23 +808,8 @@
             raise IDPresent(entry.id)
         self.entries[entry.id] = entry
 
-def my_sort(sequence, key, reverse=False):
-    """A sort function that supports supplying a key for comparison
-    
-    :param sequence: The sequence to sort
-    :param key: A callable object that returns the values to be compared
-    :param reverse: If true, sort in reverse order
-    :type reverse: bool
-    """
-    def cmp_by_key(entry_a, entry_b):
-        if reverse:
-            tmp=entry_a
-            entry_a = entry_b
-            entry_b = tmp
-        return cmp(key(entry_a), key(entry_b))
-    sequence.sort(cmp_by_key)
 
-def get_rename_entries(changeset, inventory, reverse):
+def get_rename_entries(changeset, inventory):
     """Return a list of entries that will be renamed.  Entries are sorted from
     longest to shortest source path and from shortest to longest target path.
 
@@ -885,8 +817,6 @@
     :type changeset: `Changeset`
     :param inventory: The source of current tree paths for the given ids
     :type inventory: Dictionary
-    :param reverse: If true, the changeset is being applied in reverse
-    :type reverse: bool
     :return: source entries and target entries as a tuple
     :rtype: (List, List)
     """
@@ -900,22 +830,23 @@
             return 0
         else:
             return len(path)
-    my_sort(source_entries, longest_to_shortest, reverse=True)
+    source_entries.sort(None, longest_to_shortest, True)
 
     target_entries = source_entries[:]
     # These are done from shortest to longest path, to avoid creating a
     # child before its parent has been created/renamed
     def shortest_to_longest(entry):
-        path = entry.get_new_path(inventory, changeset, reverse)
+        path = entry.get_new_path(inventory, changeset)
         if path is None:
             return 0
         else:
             return len(path)
-    my_sort(target_entries, shortest_to_longest)
+    target_entries.sort(None, shortest_to_longest)
     return (source_entries, target_entries)
 
+
 def rename_to_temp_delete(source_entries, inventory, dir, temp_dir, 
-                          conflict_handler, reverse):
+                          conflict_handler):
     """Delete and rename entries as appropriate.  Entries are renamed to temp
     names.  A map of id -> temp name (or None, for deletions) is returned.
 
@@ -925,26 +856,24 @@
     :type inventory: Dictionary
     :param dir: The directory to apply changes to
     :type dir: str
-    :param reverse: Apply changes in reverse
-    :type reverse: bool
     :return: a mapping of id to temporary name
     :rtype: Dictionary
     """
     temp_name = {}
     for i in range(len(source_entries)):
         entry = source_entries[i]
-        if entry.is_deletion(reverse):
-            path = os.path.join(dir, inventory[entry.id])
-            entry.apply(path, conflict_handler, reverse)
+        if entry.is_deletion():
+            path = pathjoin(dir, inventory[entry.id])
+            entry.apply(path, conflict_handler)
             temp_name[entry.id] = None
 
         elif entry.needs_rename():
-            if entry.is_creation(reverse):
+            if entry.is_creation():
                 continue
-            to_name = os.path.join(temp_dir, str(i))
+            to_name = pathjoin(temp_dir, str(i))
             src_path = inventory.get(entry.id)
             if src_path is not None:
-                src_path = os.path.join(dir, src_path)
+                src_path = pathjoin(dir, src_path)
                 try:
                     rename(src_path, to_name)
                     temp_name[entry.id] = to_name
@@ -959,7 +888,7 @@
 
 
 def rename_to_new_create(changed_inventory, target_entries, inventory, 
-                         changeset, dir, conflict_handler, reverse):
+                         changeset, dir, conflict_handler):
     """Rename entries with temp names to their final names, create new files.
 
     :param changed_inventory: A mapping of id to temporary name
@@ -970,24 +899,22 @@
     :type changeset: `Changeset`
     :param dir: The directory to apply changes to
     :type dir: str
-    :param reverse: If true, apply changes in reverse
-    :type reverse: bool
     """
     for entry in target_entries:
-        new_tree_path = entry.get_new_path(inventory, changeset, reverse)
+        new_tree_path = entry.get_new_path(inventory, changeset)
         if new_tree_path is None:
             continue
-        new_path = os.path.join(dir, new_tree_path)
+        new_path = pathjoin(dir, new_tree_path)
         old_path = changed_inventory.get(entry.id)
         if bzrlib.osutils.lexists(new_path):
             if conflict_handler.target_exists(entry, new_path, old_path) == \
                 "skip":
                 continue
-        if entry.is_creation(reverse):
-            entry.apply(new_path, conflict_handler, reverse)
+        if entry.is_creation():
+            entry.apply(new_path, conflict_handler)
             changed_inventory[entry.id] = new_tree_path
         elif entry.needs_rename():
-            if entry.is_deletion(reverse):
+            if entry.is_deletion():
                 continue
             if old_path is None:
                 continue
@@ -999,6 +926,7 @@
                 raise BzrCheckError('failed to rename %s to %s for changeset entry %s: %s'
                         % (old_path, new_path, entry, e))
 
+
 class TargetExists(Exception):
     def __init__(self, entry, target):
         msg = "The path %s already exists" % target
@@ -1006,6 +934,7 @@
         self.entry = entry
         self.target = target
 
+
 class RenameConflict(Exception):
     def __init__(self, id, this_name, base_name, other_name):
         msg = """Trees all have different names for a file
@@ -1018,6 +947,7 @@
         self.base_name = base_name
         self_other_name = other_name
 
+
 class MoveConflict(Exception):
     def __init__(self, id, this_parent, base_parent, other_parent):
         msg = """The file is in different directories in every tree
@@ -1030,17 +960,20 @@
         self.base_parent = base_parent
         self_other_parent = other_parent
 
+
 class MergeConflict(Exception):
     def __init__(self, this_path):
         Exception.__init__(self, "Conflict applying changes to %s" % this_path)
         self.this_path = this_path
 
+
 class WrongOldContents(Exception):
     def __init__(self, filename):
         msg = "Contents mismatch deleting %s" % filename
         self.filename = filename
         Exception.__init__(self, msg)
 
+
 class WrongOldExecFlag(Exception):
     def __init__(self, filename, old_exec_flag, new_exec_flag):
         msg = "Executable flag missmatch on %s:\n" \
@@ -1048,6 +981,7 @@
         self.filename = filename
         Exception.__init__(self, msg)
 
+
 class RemoveContentsConflict(Exception):
     def __init__(self, filename):
         msg = "Conflict deleting %s, which has different contents in BASE"\
@@ -1055,6 +989,7 @@
         self.filename = filename
         Exception.__init__(self, msg)
 
+
 class DeletingNonEmptyDirectory(Exception):
     def __init__(self, filename):
         msg = "Trying to remove dir %s while it still had files" % filename
@@ -1068,6 +1003,7 @@
         Exception.__init__(self, msg)
         self.filename = filename
 
+
 class MissingForSetExec(Exception):
     def __init__(self, filename):
         msg = "Attempt to change permissions on  %s, which does not exist" %\
@@ -1075,6 +1011,7 @@
         Exception.__init__(self, msg)
         self.filename = filename
 
+
 class MissingForRm(Exception):
     def __init__(self, filename):
         msg = "Attempt to remove missing path %s" % filename
@@ -1088,16 +1025,19 @@
         Exception.__init__(self, msg)
         self.filename = filename
 
+
 class NewContentsConflict(Exception):
     def __init__(self, filename):
         msg = "Conflicting contents for new file %s" % (filename)
         Exception.__init__(self, msg)
 
+
 class WeaveMergeConflict(Exception):
     def __init__(self, filename):
         msg = "Conflicting contents for file %s" % (filename)
         Exception.__init__(self, msg)
 
+
 class ThreewayContentsConflict(Exception):
     def __init__(self, filename):
         msg = "Conflicting contents for file %s" % (filename)
@@ -1184,8 +1124,8 @@
     def finalize(self):
         pass
 
-def apply_changeset(changeset, inventory, dir, conflict_handler=None, 
-                    reverse=False):
+
+def apply_changeset(changeset, inventory, dir, conflict_handler=None):
     """Apply a changeset to a directory.
 
     :param changeset: The changes to perform
@@ -1194,14 +1134,12 @@
     :type inventory: Dictionary
     :param dir: The path of the directory to apply the changes to
     :type dir: str
-    :param reverse: If true, apply the changes in reverse
-    :type reverse: bool
     :return: The mapping of the changed entries
     :rtype: Dictionary
     """
     if conflict_handler is None:
         conflict_handler = ExceptionConflictHandler()
-    temp_dir = os.path.join(dir, "bzr-tree-change")
+    temp_dir = pathjoin(dir, "bzr-tree-change")
     try:
         os.mkdir(temp_dir)
     except OSError, e:
@@ -1222,38 +1160,35 @@
                 warning("entry {%s} no longer present, can't be updated",
                         entry.id)
                 continue
-            path = os.path.join(dir, inventory[entry.id])
-            entry.apply(path, conflict_handler, reverse)
+            path = pathjoin(dir, inventory[entry.id])
+            entry.apply(path, conflict_handler)
 
     # Apply renames in stages, to minimize conflicts:
     # Only files whose name or parent change are interesting, because their
     # target name may exist in the source tree.  If a directory's name changes,
     # that doesn't make its children interesting.
-    (source_entries, target_entries) = get_rename_entries(changeset, inventory,
-                                                          reverse)
+    (source_entries, target_entries) = get_rename_entries(changeset, inventory)
 
     changed_inventory = rename_to_temp_delete(source_entries, inventory, dir,
-                                              temp_dir, conflict_handler,
-                                              reverse)
+                                              temp_dir, conflict_handler)
 
     rename_to_new_create(changed_inventory, target_entries, inventory,
-                         changeset, dir, conflict_handler, reverse)
+                         changeset, dir, conflict_handler)
     os.rmdir(temp_dir)
     return changed_inventory
 
 
-def apply_changeset_tree(cset, tree, reverse=False):
+def apply_changeset_tree(cset, tree):
     r_inventory = {}
     for entry in tree.source_inventory().itervalues():
         inventory[entry.id] = entry.path
-    new_inventory = apply_changeset(cset, r_inventory, tree.basedir,
-                                    reverse=reverse)
+    new_inventory = apply_changeset(cset, r_inventory, tree.basedir)
     new_entries, remove_entries = \
-        get_inventory_change(inventory, new_inventory, cset, reverse)
+        get_inventory_change(inventory, new_inventory, cset)
     tree.update_source_inventory(new_entries, remove_entries)
 
 
-def get_inventory_change(inventory, new_inventory, cset, reverse=False):
+def get_inventory_change(inventory, new_inventory, cset):
     new_entries = {}
     remove_entries = []
     for entry in cset.entries.itervalues():
@@ -1278,127 +1213,6 @@
         print entry.id
         print entry.summarize_name(cset)
 
-class CompositionFailure(Exception):
-    def __init__(self, old_entry, new_entry, problem):
-        msg = "Unable to conpose entries.\n %s" % problem
-        Exception.__init__(self, msg)
-
-class IDMismatch(CompositionFailure):
-    def __init__(self, old_entry, new_entry):
-        problem = "Attempt to compose entries with different ids: %s and %s" %\
-            (old_entry.id, new_entry.id)
-        CompositionFailure.__init__(self, old_entry, new_entry, problem)
-
-def compose_changesets(old_cset, new_cset):
-    """Combine two changesets into one.  This works well for exact patching.
-    Otherwise, not so well.
-
-    :param old_cset: The first changeset that would be applied
-    :type old_cset: `Changeset`
-    :param new_cset: The second changeset that would be applied
-    :type new_cset: `Changeset`
-    :return: A changeset that combines the changes in both changesets
-    :rtype: `Changeset`
-    """
-    composed = Changeset()
-    for old_entry in old_cset.entries.itervalues():
-        new_entry = new_cset.entries.get(old_entry.id)
-        if new_entry is None:
-            composed.add_entry(old_entry)
-        else:
-            composed_entry = compose_entries(old_entry, new_entry)
-            if composed_entry.parent is not None or\
-                composed_entry.new_parent is not None:
-                composed.add_entry(composed_entry)
-    for new_entry in new_cset.entries.itervalues():
-        if not old_cset.entries.has_key(new_entry.id):
-            composed.add_entry(new_entry)
-    return composed
-
-def compose_entries(old_entry, new_entry):
-    """Combine two entries into one.
-
-    :param old_entry: The first entry that would be applied
-    :type old_entry: ChangesetEntry
-    :param old_entry: The second entry that would be applied
-    :type old_entry: ChangesetEntry
-    :return: A changeset entry combining both entries
-    :rtype: `ChangesetEntry`
-    """
-    if old_entry.id != new_entry.id:
-        raise IDMismatch(old_entry, new_entry)
-    output = ChangesetEntry(old_entry.id, old_entry.parent, old_entry.path)
-
-    if (old_entry.parent != old_entry.new_parent or 
-        new_entry.parent != new_entry.new_parent):
-        output.new_parent = new_entry.new_parent
-
-    if (old_entry.path != old_entry.new_path or 
-        new_entry.path != new_entry.new_path):
-        output.new_path = new_entry.new_path
-
-    output.contents_change = compose_contents(old_entry, new_entry)
-    output.metadata_change = compose_metadata(old_entry, new_entry)
-    return output
-
-def compose_contents(old_entry, new_entry):
-    """Combine the contents of two changeset entries.  Entries are combined
-    intelligently where possible, but the fallback behavior returns an 
-    ApplySequence.
-
-    :param old_entry: The first entry that would be applied
-    :type old_entry: `ChangesetEntry`
-    :param new_entry: The second entry that would be applied
-    :type new_entry: `ChangesetEntry`
-    :return: A combined contents change
-    :rtype: anything supporting the apply(reverse=False) method
-    """
-    old_contents = old_entry.contents_change
-    new_contents = new_entry.contents_change
-    if old_entry.contents_change is None:
-        return new_entry.contents_change
-    elif new_entry.contents_change is None:
-        return old_entry.contents_change
-    elif isinstance(old_contents, ReplaceContents) and \
-        isinstance(new_contents, ReplaceContents):
-        if old_contents.old_contents == new_contents.new_contents:
-            return None
-        else:
-            return ReplaceContents(old_contents.old_contents,
-                                   new_contents.new_contents)
-    elif isinstance(old_contents, ApplySequence):
-        output = ApplySequence(old_contents.changes)
-        if isinstance(new_contents, ApplySequence):
-            output.changes.extend(new_contents.changes)
-        else:
-            output.changes.append(new_contents)
-        return output
-    elif isinstance(new_contents, ApplySequence):
-        output = ApplySequence((old_contents.changes,))
-        output.extend(new_contents.changes)
-        return output
-    else:
-        return ApplySequence((old_contents, new_contents))
-
-def compose_metadata(old_entry, new_entry):
-    old_meta = old_entry.metadata_change
-    new_meta = new_entry.metadata_change
-    if old_meta is None:
-        return new_meta
-    elif new_meta is None:
-        return old_meta
-    elif (isinstance(old_meta, ChangeExecFlag) and
-          isinstance(new_meta, ChangeExecFlag)):
-        return ChangeExecFlag(old_meta.old_exec_flag, new_meta.new_exec_flag)
-    else:
-        return ApplySequence(old_meta, new_meta)
-
-
-def changeset_is_null(changeset):
-    for entry in changeset.entries.itervalues():
-        if not entry.is_boring():
-            return False
-    return True
 
 class UnsupportedFiletype(Exception):
     def __init__(self, kind, full_path):
@@ -1408,6 +1222,7 @@
         self.full_path = full_path
         self.kind = kind
 
+
 def generate_changeset(tree_a, tree_b, interesting_ids=None):
     return ChangesetGenerator(tree_a, tree_b, interesting_ids)()
 
@@ -1502,7 +1317,6 @@
             return self.make_entry(id, only_interesting=False)
         else:
             return cs_entry
-        
 
     def make_entry(self, id, only_interesting=True):
         cs_entry = self.make_basic_entry(id, only_interesting)
@@ -1557,7 +1371,8 @@
 
 
 def full_path(entry, tree):
-    return os.path.join(tree.basedir, entry.path)
+    return pathjoin(tree.basedir, entry.path)
+
 
 def new_delete_entry(entry, tree, inventory, delete):
     if entry.path == "":
@@ -1575,10 +1390,8 @@
     status = os.lstat(full_path)
     if stat.S_ISDIR(file_stat.st_mode):
         action = dir_create
-    
-
-
-        
+
+
 # XXX: Can't we unify this with the regular inventory object
 class Inventory(object):
     def __init__(self, inventory):
@@ -1617,3 +1430,5 @@
         if directory is None:
             return NULL_ID
         return self.get_rinventory().get(directory)
+
+

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2005-11-28 05:13:41 +0000
+++ bzrlib/check.py	2005-12-19 20:31:01 +0000
@@ -50,6 +50,7 @@
         self.missing_revision_cnt = 0
         # maps (file-id, version) -> sha1
         self.checked_texts = {}
+        self.checked_weaves = {}
 
     def check(self):
         self.branch.lock_read()
@@ -65,6 +66,7 @@
                 return
             self.plan_revisions()
             revno = 0
+            self.check_weaves()
             while revno < len(self.planned_revisions):
                 rev_id = self.planned_revisions[revno]
                 self.progress.update('checking revision', revno,
@@ -101,6 +103,7 @@
         note('%6d revisions', self.checked_rev_cnt)
         note('%6d unique file texts', self.checked_text_cnt)
         note('%6d repeated file texts', self.repeated_text_cnt)
+        note('%6d weaves', len(self.checked_weaves))
         if self.missing_inventory_sha_cnt:
             note('%6d revisions are missing inventory_sha1',
                  self.missing_inventory_sha_cnt)
@@ -188,6 +191,24 @@
         self._check_revision_tree(rev_id)
         self.checked_rev_cnt += 1
 
+    def check_weaves(self):
+        """Check all the weaves we can get our hands on.
+        """
+        n_weaves = 1
+        weave_ids = []
+        if self.branch.weave_store.listable():
+            weave_ids = list(self.branch.weave_store)
+            n_weaves = len(weave_ids)
+        self.progress.update('checking weave', 0, n_weaves)
+        self.inventory_weave.check(progress_bar=self.progress)
+        for i, weave_id in enumerate(weave_ids):
+            self.progress.update('checking weave', i, n_weaves)
+            w = self.branch.weave_store.get_weave(weave_id,
+                    self.branch.get_transaction())
+            # No progress here, because it looks ugly.
+            w.check()
+            self.checked_weaves[weave_id] = True
+
     def _check_revision_tree(self, rev_id):
         tree = self.branch.revision_tree(rev_id)
         inv = tree.inventory

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2005-11-21 09:07:18 +0000
+++ bzrlib/config.py	2006-01-02 22:37:32 +0000
@@ -60,6 +60,8 @@
 
 import bzrlib
 import bzrlib.errors as errors
+from bzrlib.osutils import pathjoin
+from bzrlib.trace import mutter
 import bzrlib.util.configobj.configobj as configobj
 from StringIO import StringIO
 
@@ -348,8 +350,8 @@
         """Save option and its value in the configuration."""
         # FIXME: RBC 20051029 This should refresh the parser and also take a
         # file lock on branches.conf.
-        if not os.path.isdir(os.path.dirname(self._get_filename())):
-            os.mkdir(os.path.dirname(self._get_filename()))
+        conf_dir = os.path.dirname(self._get_filename())
+        ensure_config_dir_exists(conf_dir)
         location = self.location
         if location.endswith('/'):
             location = location[:-1]
@@ -408,6 +410,24 @@
         return self._get_location_config()._post_commit()
 
 
+def ensure_config_dir_exists(path=None):
+    """Make sure a configuration directory exists.
+    This makes sure that the directory exists.
+    On windows, since configuration directories are 2 levels deep,
+    it makes sure both the directory and the parent directory exists.
+    """
+    if path is None:
+        path = config_dir()
+    if not os.path.isdir(path):
+        if sys.platform == 'win32':
+            parent_dir = os.path.dirname(path)
+            if not os.path.isdir(parent_dir):
+                mutter('creating config parent directory: %r', parent_dir)
+            os.mkdir(parent_dir)
+        mutter('creating config directory: %r', path)
+        os.mkdir(path)
+
+
 def config_dir():
     """Return per-user configuration directory.
 
@@ -423,22 +443,22 @@
             base = os.environ.get('HOME', None)
         if base is None:
             raise BzrError('You must have one of BZR_HOME, APPDATA, or HOME set')
-        return os.path.join(base, 'bazaar', '2.0')
+        return pathjoin(base, 'bazaar', '2.0')
     else:
         # cygwin, linux, and darwin all have a $HOME directory
         if base is None:
             base = os.path.expanduser("~")
-        return os.path.join(base, ".bazaar")
+        return pathjoin(base, ".bazaar")
 
 
 def config_filename():
     """Return per-user configuration ini file filename."""
-    return os.path.join(config_dir(), 'bazaar.conf')
+    return pathjoin(config_dir(), 'bazaar.conf')
 
 
 def branches_config_filename():
     """Return per-user configuration ini file filename."""
-    return os.path.join(config_dir(), 'branches.conf')
+    return pathjoin(config_dir(), 'branches.conf')
 
 
 def _auto_user_id():

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2005-11-30 00:25:50 +0000
+++ bzrlib/conflicts.py	2005-12-01 23:52:16 +0000
@@ -30,6 +30,7 @@
 from bzrlib.errors import BzrCommandError, NotConflicted
 from bzrlib.commands import register_command
 from bzrlib.workingtree import CONFLICT_SUFFIXES
+from bzrlib.osutils import rename
 
 class cmd_conflicts(bzrlib.commands.Command):
     """List files with conflicts.
@@ -80,7 +81,7 @@
     """
     conflicted = False
     try:
-        os.rename(filename + ".THIS", filename)
+        rename(filename + ".THIS", filename)
         conflicted = True
     except OSError, e:
         if e.errno != errno.ENOENT:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-12-31 15:19:21 +0000
+++ bzrlib/errors.py	2006-01-03 23:51:45 +0000
@@ -34,7 +34,9 @@
 ... except:
 ...   print sys.exc_type
 ...   print sys.exc_value
-...   print sys.exc_value.path
+...   path = getattr(sys.exc_value, 'path')
+...   if path is not None:
+...     print path
 bzrlib.errors.NotBranchError
 Not a branch: /foo/bar
 /foo/bar
@@ -411,6 +413,10 @@
     """Parents are mismatched between two revisions."""
     
 
+class WeaveInvalidChecksum(WeaveError):
+    """Text did not match it's checksum: %(message)s"""
+
+
 class NoSuchExportFormat(BzrNewError):
     """Export format %(format)r not supported"""
     def __init__(self, format):

=== modified file 'bzrlib/export/dir_exporter.py'
--- bzrlib/export/dir_exporter.py	2005-11-14 23:58:51 +0000
+++ bzrlib/export/dir_exporter.py	2005-12-29 03:26:31 +0000
@@ -37,5 +37,10 @@
     mutter('export version %r', tree)
     inv = tree.inventory
     for dp, ie in inv.iter_entries():
+        # .bzrignore has no meaning outside of a working tree
+        # so do not export it
+        if dp == ".bzrignore":
+            continue
+        
         ie.put_on_disk(dest, dp, tree)
 

=== modified file 'bzrlib/export/tar_exporter.py'
--- bzrlib/export/tar_exporter.py	2005-11-14 23:58:51 +0000
+++ bzrlib/export/tar_exporter.py	2005-12-29 03:26:31 +0000
@@ -40,6 +40,11 @@
     mutter('export version %r', tree)
     inv = tree.inventory
     for dp, ie in inv.iter_entries():
+        # .bzrignore has no meaning outside of a working tree
+        # so do not export it
+        if dp == ".bzrignore":
+            continue
+        
         mutter("  export {%s} kind %s to %s", ie.file_id, ie.kind, dest)
         item, fileobj = ie.get_tar_item(root, dp, now, tree)
         ball.addfile(item, fileobj)

=== modified file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	2005-11-14 23:58:51 +0000
+++ bzrlib/export/zip_exporter.py	2006-01-02 21:20:36 +0000
@@ -39,6 +39,10 @@
 
     try:
         for dp, ie in inv.iter_entries():
+            # .bzrignore has no meaning outside of a working tree
+            # so do not export it
+            if dp == ".bzrignore":
+                continue
 
             file_id = ie.file_id
             mutter("  export {%s} kind %s to %s", file_id, ie.kind, dest)

=== modified file 'bzrlib/externalcommand.py'
--- bzrlib/externalcommand.py	2005-11-09 18:30:20 +0000
+++ bzrlib/externalcommand.py	2005-12-01 23:52:16 +0000
@@ -23,6 +23,7 @@
 import sys
 
 from bzrlib.commands import Command
+from bzrlib.osutils import pathjoin
 
 
 class ExternalCommand(Command):
@@ -37,6 +38,8 @@
             ## Empty directories are not real paths
             if not dir:
                 continue
+            # This needs to be os.path.join() or windows cannot
+            # find the batch file that you are wanting to execute
             path = os.path.join(dir, cmd)
             if os.path.isfile(path):
                 return ExternalCommand(path)
@@ -47,20 +50,17 @@
     def __init__(self, path):
         self.path = path
 
-
     def name(self):
-        return self.path.split(os.sep)[-1]
-
+        return os.path.basename(self.path)
 
     def run(self, *args, **kwargs):
         raise NotImplementedError('should not be called on %r' % self)
 
-
     def run_argv(self, argv):
         return os.spawnv(os.P_WAIT, self.path, [self.path] + argv)
 
-
     def help(self):
         m = 'external command from %s\n\n' % self.path
         pipe = os.popen('%s --help' % self.path)
         return m + pipe.read()
+

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2005-11-10 03:03:53 +0000
+++ bzrlib/hashcache.py	2005-12-01 17:36:29 +0000
@@ -35,6 +35,7 @@
 from bzrlib.osutils import sha_file
 from bzrlib.trace import mutter, warning
 from bzrlib.atomicfile import AtomicFile
+from bzrlib.osutils import pathjoin
 
 
 FP_MODE_COLUMN = 5
@@ -104,7 +105,7 @@
     def cache_file_name(self):
         # FIXME: duplicate path logic here, this should be 
         # something like 'branch.controlfile'.
-        return os.sep.join([self.basedir, '.bzr', 'stat-cache'])
+        return pathjoin(self.basedir, '.bzr', 'stat-cache')
 
     def clear(self):
         """Discard all cached information.
@@ -125,7 +126,7 @@
         prep.sort()
         
         for inum, path, cache_entry in prep:
-            abspath = os.sep.join([self.basedir, path])
+            abspath = pathjoin(self.basedir, path)
             fp = _fingerprint(abspath)
             self.stat_count += 1
             
@@ -141,7 +142,7 @@
     def get_sha1(self, path):
         """Return the sha1 of a file.
         """
-        abspath = os.sep.join([self.basedir, path])
+        abspath = pathjoin(self.basedir, path)
         self.stat_count += 1
         file_fp = _fingerprint(abspath)
         

=== modified file 'bzrlib/intset.py'
--- bzrlib/intset.py	2005-09-19 06:05:19 +0000
+++ bzrlib/intset.py	2005-12-19 17:48:25 +0000
@@ -208,3 +208,22 @@
         if not self._val & m:
             raise KeyError(to_remove)
         self._val ^= m
+
+    def set_remove(self, to_remove):
+        """Remove all values that exist in to_remove.
+
+        >>> a = IntSet(range(10))
+        >>> b = IntSet([2,3,4,7,12])
+        >>> a.set_remove(b)
+        >>> list(a)
+        [0, 1, 5, 6, 8, 9]
+        >>> a.set_remove([1,2,5])
+        >>> list(a)
+        [0, 6, 8, 9]
+        """
+        if not isinstance(to_remove, IntSet):
+            self.set_remove(IntSet(to_remove))
+            return
+        intersect = self._val & to_remove._val
+        self._val ^= intersect
+

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-11-30 00:25:50 +0000
+++ bzrlib/inventory.py	2005-12-19 20:31:01 +0000
@@ -36,7 +36,7 @@
 
 import bzrlib
 from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
-                            appendpath, sha_strings)
+                            pathjoin, sha_strings)
 from bzrlib.trace import mutter
 from bzrlib.errors import (NotVersionedError, InvalidEntryName,
                            BzrError, BzrCheckError)
@@ -79,7 +79,7 @@
     InventoryDirectory('123', 'src', parent_id='TREE_ROOT')
     >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
     InventoryFile('2323', 'hello.c', parent_id='123')
-    >>> shouldbe = {0: 'src', 1: os.path.join('src','hello.c')}
+    >>> shouldbe = {0: 'src', 1: pathjoin('src','hello.c')}
     >>> for ix, j in enumerate(i.iter_entries()):
     ...   print (j[0] == shouldbe[ix], j[1])
     ... 
@@ -102,7 +102,7 @@
     >>> i['2326']
     InventoryFile('2326', 'wibble.c', parent_id='2325')
     >>> for path, entry in i.iter_entries():
-    ...     print path.replace('\\\\', '/')     # for win32 os.sep
+    ...     print path
     ...     assert i.path2id(path)
     ... 
     src
@@ -110,7 +110,7 @@
     src/hello.c
     src/wibble
     src/wibble/wibble.c
-    >>> i.id2path('2326').replace('\\\\', '/')
+    >>> i.id2path('2326')
     'src/wibble/wibble.c'
     """
     
@@ -202,7 +202,7 @@
 
     def get_tar_item(self, root, dp, now, tree):
         """Get a tarfile item and a file stream for its content."""
-        item = tarfile.TarInfo(os.path.join(root, dp))
+        item = tarfile.TarInfo(pathjoin(root, dp))
         # TODO: would be cool to actually set it to the timestamp of the
         # revision it was last changed
         item.mtime = now
@@ -267,7 +267,7 @@
         
         This is a template method - implement _put_on_disk in subclasses.
         """
-        fullpath = appendpath(dest, dp)
+        fullpath = pathjoin(dest, dp)
         self._put_on_disk(fullpath, tree)
         mutter("  export {%s} kind %s to %s", self.file_id,
                 self.kind, fullpath)
@@ -482,13 +482,26 @@
             else:
                 checker.repeated_text_cnt += 1
                 return
+
+        if self.file_id not in checker.checked_weaves:
+            mutter('check weave {%s}', self.file_id)
+            w = tree.get_weave(self.file_id)
+            # Not passing a progress bar, because it creates a new
+            # progress, which overwrites the current progress,
+            # and doesn't look nice
+            w.check()
+            checker.checked_weaves[self.file_id] = True
+        else:
+            w = tree.get_weave_prelude(self.file_id)
+
         mutter('check version {%s} of {%s}', rev_id, self.file_id)
-        file_lines = tree.get_file_lines(self.file_id)
         checker.checked_text_cnt += 1 
-        if self.text_size != sum(map(len, file_lines)):
-            raise BzrCheckError('text {%s} wrong size' % self.text_id)
-        if self.text_sha1 != sha_strings(file_lines):
-            raise BzrCheckError('text {%s} wrong sha1' % self.text_id)
+        # We can't check the length, because Weave doesn't store that
+        # information, and the whole point of looking at the weave's
+        # sha1sum is that we don't have to extract the text.
+        if self.text_sha1 != w.get_sha1(self.revision):
+            raise BzrCheckError('text {%s} version {%s} wrong sha1' 
+                                % (self.file_id, self.revision))
         checker.checked_texts[t] = self.text_sha1
 
     def copy(self):
@@ -764,7 +777,7 @@
             yield name, ie
             if ie.kind == 'directory':
                 for cn, cie in self.iter_entries(from_dir=ie.file_id):
-                    yield os.path.join(name, cn), cie
+                    yield pathjoin(name, cn), cie
 
 
     def entries(self):
@@ -777,7 +790,7 @@
             kids = dir_ie.children.items()
             kids.sort()
             for name, ie in kids:
-                child_path = os.path.join(dir_path, name)
+                child_path = pathjoin(dir_path, name)
                 accum.append((child_path, ie))
                 if ie.kind == 'directory':
                     descend(ie, child_path)
@@ -797,7 +810,7 @@
             kids.sort()
 
             for name, child_ie in kids:
-                child_path = os.path.join(parent_path, name)
+                child_path = pathjoin(parent_path, name)
                 descend(child_ie, child_path)
         descend(self.root, u'')
         return accum
@@ -864,7 +877,7 @@
 
         if parent.children.has_key(entry.name):
             raise BzrError("%s is already versioned" %
-                    appendpath(self.id2path(parent.file_id), entry.name))
+                    pathjoin(self.id2path(parent.file_id), entry.name))
 
         self._byid[entry.file_id] = entry
         parent.children[entry.name] = entry
@@ -987,12 +1000,15 @@
         >>> i = Inventory()
         >>> e = i.add(InventoryDirectory('src-id', 'src', ROOT_ID))
         >>> e = i.add(InventoryFile('foo-id', 'foo.c', parent_id='src-id'))
-        >>> print i.id2path('foo-id').replace(os.sep, '/')
+        >>> print i.id2path('foo-id')
         src/foo.c
         """
         # get all names, skipping root
         p = [self._byid[fid].name for fid in self.get_idpath(file_id)[1:]]
-        return os.sep.join(p)
+        if p:
+            return pathjoin(*p)
+        else:
+            return ''
             
 
 

=== modified file 'bzrlib/merge.py' (properties changed: -x to +x)
--- bzrlib/merge.py	2005-12-02 17:18:17 +0000
+++ bzrlib/merge.py	2005-12-22 23:25:30 +0000
@@ -16,7 +16,6 @@
 
 
 import os
-import tempfile
 import shutil
 import errno
 
@@ -39,7 +38,7 @@
 from bzrlib.trace import mutter, warning, note
 from bzrlib.fetch import greedy_fetch, fetch
 from bzrlib.revision import is_ancestor, NULL_REVISION
-from bzrlib.osutils import rename
+from bzrlib.osutils import rename, pathjoin
 from bzrlib.revision import common_ancestor, MultipleRevisionSources
 from bzrlib.errors import NoSuchRevision
 
@@ -172,13 +171,8 @@
         return "skip"
 
     def rem_contents_conflict(self, filename, this_contents, base_contents):
-        base_contents(filename+".BASE", self, False)
-        this_contents(filename+".THIS", self, False)
-        return ReplaceContents(this_contents, None)
-
-    def rem_contents_conflict(self, filename, this_contents, base_contents):
-        base_contents(filename+".BASE", self, False)
-        this_contents(filename+".THIS", self, False)
+        base_contents(filename+".BASE", self)
+        this_contents(filename+".THIS", self)
         self.conflict("Other branch deleted locally modified file %s" %
                       filename)
         return ReplaceContents(this_contents, None)
@@ -202,22 +196,22 @@
             abspath = self.create_all_missing(entry.parent_id, tree)
         else:
             abspath = self.abs_this_path(entry.parent_id)
-        entry_path = os.path.join(abspath, entry.name)
+        entry_path = pathjoin(abspath, entry.name)
         if not os.path.isdir(entry_path):
             self.create(file_id, entry_path, tree)
         return entry_path
 
-    def create(self, file_id, path, tree, reverse=False):
+    def create(self, file_id, path, tree):
         """Uses tree data to create a filesystem object for the file_id"""
         from changeset import get_contents
-        get_contents(tree, file_id)(path, self, reverse)
+        get_contents(tree, file_id)(path, self)
 
     def missing_for_merge(self, file_id, other_path):
         """The file_id doesn't exist in THIS, but does in OTHER and BASE"""
         self.conflict("Other branch modified locally deleted file %s" %
                       other_path)
         parent_dir = self.add_missing_parents(file_id, self.other_tree)
-        stem = os.path.join(parent_dir, os.path.basename(other_path))
+        stem = pathjoin(parent_dir, os.path.basename(other_path))
         self.create(file_id, stem+".OTHER", self.other_tree)
         self.create(file_id, stem+".BASE", self.base_tree)
 
@@ -586,7 +580,7 @@
             entry = old_entries[file_id]
             if entry.parent_id is None:
                 return entry.name
-            return os.path.join(id2path(entry.parent_id), entry.name)
+            return pathjoin(id2path(entry.parent_id), entry.name)
             
         for file_id in old_entries:
             entry = old_entries[file_id]
@@ -613,7 +607,7 @@
                 parent = None
             else:
                 parent = by_path[os.path.dirname(path)]
-            abspath = os.path.join(self.this_tree.basedir, path)
+            abspath = pathjoin(self.this_tree.basedir, path)
             kind = bzrlib.osutils.file_kind(abspath)
             new_inventory[file_id] = (path, file_id, parent, kind)
             by_path[path] = file_id 

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-11-10 03:03:53 +0000
+++ bzrlib/merge_core.py	2005-12-22 23:32:43 +0000
@@ -2,15 +2,18 @@
 
 import changeset
 from changeset import Inventory, apply_changeset, invert_dict
-from bzrlib.osutils import backup_file, rename
+from bzrlib.osutils import backup_file, rename, pathjoin
 from bzrlib.merge3 import Merge3
 import bzrlib
 from bzrlib.atomicfile import AtomicFile
 from changeset import get_contents
 
+
 class ApplyMerge3:
+    """Contents-change wrapper around merge3.Merge3"""
+
     history_based = False
-    """Contents-change wrapper around merge3.Merge3"""
+
     def __init__(self, file_id, base, other, show_base=False, reprocess=False):
         self.file_id = file_id
         self.base = base
@@ -33,14 +36,10 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         new_file = filename+".new" 
-        if not reverse:
-            base = self.base
-            other = self.other
-        else:
-            base = self.other
-            other = self.base
+        base = self.base
+        other = self.other
         def get_lines(tree):
             if self.file_id not in tree:
                 raise Exception("%s not in tree" % self.file_id)
@@ -75,9 +74,12 @@
             conflict_handler.merge_conflict(new_file, filename, base_lines,
                                             other_lines)
 
+
 class WeaveMerge:
     """Contents-change wrapper around weave merge"""
+
     history_based = True
+
     def __init__(self, weave, this_revision_id, other_revision_id):
         self.weave = weave
         self.this_revision_id = this_revision_id
@@ -99,7 +101,7 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         this_i = self.weave.lookup(self.this_revision_id)
         other_i = self.weave.lookup(self.other_revision_id)
         plan = self.weave.plan_merge(this_i, other_i)
@@ -116,8 +118,10 @@
         else:
             out_file.commit()
 
+
 class BackupBeforeChange:
     """Contents-change wrapper to back up file first"""
+
     def __init__(self, contents_change):
         self.contents_change = contents_change
 
@@ -135,9 +139,9 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         backup_file(filename)
-        self.contents_change.apply(filename, conflict_handler, reverse)
+        self.contents_change.apply(filename, conflict_handler)
 
 
 def invert_invent(inventory):
@@ -158,7 +162,7 @@
     new_cset = make_merge_changeset(cset, this, base, other, 
                                     conflict_handler, merge_factory)
     result = apply_changeset(new_cset, invert_invent(this.inventory),
-                             this.basedir, conflict_handler, False)
+                             this.basedir, conflict_handler)
     return result
     
 
@@ -181,6 +185,7 @@
 
     return new_cset
 
+
 class ThreeWayConflict(Exception):
     def __init__(self, this, base, other):
         self.this = this
@@ -189,6 +194,7 @@
         msg = "Conflict merging %s %s and %s" % (this, base, other)
         Exception.__init__(self, msg)
 
+
 def threeway_select(this, base, other):
     """Returns a value selected by the three-way algorithm.
     Raises ThreewayConflict if the algorithm yields a conflict"""
@@ -239,7 +245,7 @@
             parent_dir = {this_parent: this_dir, other_parent: other_dir, 
                           base_parent: base_dir}
             directory = parent_dir[parent]
-            return os.path.join(directory, name)
+            return pathjoin(directory, name)
         else:
             assert parent is None
             return None
@@ -329,13 +335,9 @@
         self.other_tree = other_tree
         self.file_id = file_id
 
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            base = self.base_tree
-            other = self.other_tree
-        else:
-            base = self.other_tree
-            other = self.base_tree
+    def apply(self, filename, conflict_handler):
+        base = self.base_tree
+        other = self.other_tree
         base_exec_flag = base.is_executable(self.file_id)
         other_exec_flag = other.is_executable(self.file_id)
         this_mode = os.stat(filename).st_mode
@@ -357,3 +359,4 @@
                 to_mode = current_mode & ~0111
             os.chmod(filename, to_mode)
 
+

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-12-01 21:31:42 +0000
+++ bzrlib/osutils.py	2006-01-03 15:43:33 +0000
@@ -28,9 +28,10 @@
 import sys
 import time
 import types
+import tempfile
 
 import bzrlib
-from bzrlib.errors import BzrError, PathNotChild
+from bzrlib.errors import BzrError, PathNotChild, NoSuchFile
 from bzrlib.trace import mutter
 
 
@@ -97,6 +98,8 @@
         raise BzrError('invalid file kind %r' % kind)
 
 def lexists(f):
+    if hasattr(os.path, 'lexists'):
+        return os.path.lexists(f)
     try:
         if hasattr(os, 'lstat'):
             os.lstat(f)
@@ -109,16 +112,70 @@
         else:
             raise BzrError("lstat/stat of (%r): %r" % (f, e))
 
-def normalizepath(f):
-    if hasattr(os.path, 'realpath'):
-        F = os.path.realpath
-    else:
-        F = os.path.abspath
-    [p,e] = os.path.split(f)
-    if e == "" or e == "." or e == "..":
-        return F(f)
-    else:
-        return os.path.join(F(p), e)
+def fancy_rename(old, new, rename_func, unlink_func):
+    """A fancy rename, when you don't have atomic rename.
+    
+    :param old: The old path, to rename from
+    :param new: The new path, to rename to
+    :param rename_func: The potentially non-atomic rename function
+    :param unlink_func: A way to delete the target file if the full rename succeeds
+    """
+
+    # sftp rename doesn't allow overwriting, so play tricks:
+    import random
+    base = os.path.basename(new)
+    dirname = os.path.dirname(new)
+    tmp_name = u'tmp.%s.%.9f.%d.%d' % (base, time.time(), os.getpid(), random.randint(0, 0x7FFFFFFF))
+    tmp_name = pathjoin(dirname, tmp_name)
+
+    # Rename the file out of the way, but keep track if it didn't exist
+    # We don't want to grab just any exception
+    # something like EACCES should prevent us from continuing
+    # The downside is that the rename_func has to throw an exception
+    # with an errno = ENOENT, or NoSuchFile
+    file_existed = False
+    try:
+        rename_func(new, tmp_name)
+    except (NoSuchFile,), e:
+        pass
+    except IOError, e:
+        # RBC 20060103 abstraction leakage: the paramiko SFTP clients rename
+        # function raises an IOError with errno == None when a rename fails.
+        # This then gets caught here.
+        if e.errno not in (None, errno.ENOENT, errno.ENOTDIR):
+            raise
+    except Exception, e:
+        if (not hasattr(e, 'errno') 
+            or e.errno not in (errno.ENOENT, errno.ENOTDIR)):
+            raise
+    else:
+        file_existed = True
+
+    success = False
+    try:
+        # This may throw an exception, in which case success will
+        # not be set.
+        rename_func(old, new)
+        success = True
+    finally:
+        if file_existed:
+            # If the file used to exist, rename it back into place
+            # otherwise just delete it from the tmp location
+            if success:
+                unlink_func(tmp_name)
+            else:
+                rename_func(tmp_name, new)
+
+# Default is to just use the python builtins
+abspath = os.path.abspath
+realpath = os.path.realpath
+pathjoin = os.path.join
+normpath = os.path.normpath
+getcwd = os.getcwdu
+mkdtemp = tempfile.mkdtemp
+rename = os.rename
+dirname = os.path.dirname
+basename = os.path.basename
 
 if os.name == "posix":
     # In Python 2.4.2 and older, os.path.abspath and os.path.realpath
@@ -128,13 +185,46 @@
     _fs_enc = sys.getfilesystemencoding()
     def abspath(path):
         return os.path.abspath(path.encode(_fs_enc)).decode(_fs_enc)
+
     def realpath(path):
         return os.path.realpath(path.encode(_fs_enc)).decode(_fs_enc)
-else:
+
+if sys.platform == 'win32':
     # We need to use the Unicode-aware os.path.abspath and
     # os.path.realpath on Windows systems.
-    abspath = os.path.abspath
-    realpath = os.path.realpath
+    def abspath(path):
+        return os.path.abspath(path).replace('\\', '/')
+
+    def realpath(path):
+        return os.path.realpath(path).replace('\\', '/')
+
+    def pathjoin(*args):
+        return os.path.join(*args).replace('\\', '/')
+
+    def normpath(path):
+        return os.path.normpath(path).replace('\\', '/')
+
+    def getcwd():
+        return os.getcwdu().replace('\\', '/')
+
+    def mkdtemp(*args, **kwargs):
+        return tempfile.mkdtemp(*args, **kwargs).replace('\\', '/')
+
+    def rename(old, new):
+        fancy_rename(old, new, rename_func=os.rename, unlink_func=os.unlink)
+
+
+def normalizepath(f):
+    if hasattr(os.path, 'realpath'):
+        F = realpath
+    else:
+        F = abspath
+    [p,e] = os.path.split(f)
+    if e == "" or e == "." or e == "..":
+        return F(f)
+    else:
+        return pathjoin(F(p), e)
+
 
 def backup_file(fn):
     """Copy a file to a backup.
@@ -163,12 +253,6 @@
     finally:
         outf.close()
 
-if os.name == 'nt':
-    import shutil
-    rename = shutil.move
-else:
-    rename = os.rename
-
 
 def isdir(f):
     """True if f is an accessible directory."""
@@ -195,18 +279,18 @@
 def is_inside(dir, fname):
     """True if fname is inside dir.
     
-    The parameters should typically be passed to os.path.normpath first, so
+    The parameters should typically be passed to osutils.normpath first, so
     that . and .. and repeated slashes are eliminated, and the separators
     are canonical for the platform.
     
     The empty string as a dir name is taken as top-of-tree and matches 
     everything.
     
-    >>> is_inside('src', os.path.join('src', 'foo.c'))
+    >>> is_inside('src', pathjoin('src', 'foo.c'))
     True
     >>> is_inside('src', 'srccontrol')
     False
-    >>> is_inside('src', os.path.join('src', 'a', 'a', 'a', 'foo.c'))
+    >>> is_inside('src', pathjoin('src', 'a', 'a', 'a', 'foo.c'))
     True
     >>> is_inside('foo.c', 'foo.c')
     True
@@ -223,8 +307,8 @@
     if dir == '':
         return True
 
-    if dir[-1] != os.sep:
-        dir += os.sep
+    if dir[-1] != '/':
+        dir += '/'
 
     return fname.startswith(dir)
 
@@ -407,14 +491,14 @@
     for f in p:
         if (f == '..') or (f == None) or (f == ''):
             raise BzrError("sorry, %r not allowed in path" % f)
-    return os.path.join(*p)
+    return pathjoin(*p)
 
 
 def appendpath(p1, p2):
     if p1 == '':
         return p2
     else:
-        return os.path.join(p1, p2)
+        return pathjoin(p1, p2)
     
 
 def split_lines(s):
@@ -488,8 +572,10 @@
         # to branch anymore.
         raise PathNotChild(rp, base)
 
-    return os.sep.join(s)
-
+    if s:
+        return pathjoin(*s)
+    else:
+        return ''
 
 
 def terminal_width():

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2005-11-29 23:20:31 +0000
+++ bzrlib/plugin.py	2005-12-16 03:43:14 +0000
@@ -50,8 +50,9 @@
         log_exception_quietly
 from bzrlib.errors import BzrError
 from bzrlib import plugins
+from bzrlib.osutils import pathjoin
 
-DEFAULT_PLUGIN_PATH = os.path.join(config_dir(), 'plugins')
+DEFAULT_PLUGIN_PATH = pathjoin(config_dir(), 'plugins')
 
 _loaded = False
 
@@ -115,12 +116,12 @@
         if not os.path.isdir(d):
             continue
         for f in os.listdir(d):
-            path = os.path.join(d, f)
+            path = pathjoin(d, f)
             if os.path.isdir(path):
                 for entry in package_entries:
                     # This directory should be a package, and thus added to
                     # the list
-                    if os.path.isfile(os.path.join(path, entry)):
+                    if os.path.isfile(pathjoin(path, entry)):
                         break
                 else: # This directory is not a package
                     continue

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2005-10-17 23:13:00 +0000
+++ bzrlib/revisionspec.py	2005-12-30 20:54:47 +0000
@@ -204,7 +204,7 @@
         try:
             return RevisionInfo(branch, revs.index(self.spec) + 1, self.spec)
         except ValueError:
-            return RevisionInfo(branch, None)
+            return RevisionInfo(branch, None, self.spec)
 
 SPEC_TYPES.append(RevisionSpec_revid)
 

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2005-11-30 00:25:50 +0000
+++ bzrlib/store/__init__.py	2005-12-15 21:39:34 +0000
@@ -220,7 +220,8 @@
                 pass
         raise KeyError(fileid)
 
-    def __init__(self, a_transport, prefixed=False, compressed=False):
+    def __init__(self, a_transport, prefixed=False, compressed=False,
+                 dir_mode=None, file_mode=None):
         assert isinstance(a_transport, transport.Transport)
         super(TransportStore, self).__init__()
         self._transport = a_transport
@@ -228,6 +229,11 @@
         self._compressed = compressed
         self._suffixes = set()
 
+        # It is okay for these to be None, it just means they
+        # will just use the filesystem defaults
+        self._dir_mode = dir_mode
+        self._file_mode = file_mode
+
     def _iter_files_recursive(self):
         """Iterate through the files in the transport."""
         for quoted_relpath in self._transport.iter_files_recursive():
@@ -303,37 +309,6 @@
     return bzrlib.store.text.TextStore(transport.memory.MemoryTransport())
         
 
-class CachedStore(Store):
-    """A store that caches data locally, to avoid repeated downloads.
-    The precacache method should be used to avoid server round-trips for
-    every piece of data.
-    """
-
-    def __init__(self, store, cache_dir):
-        super(CachedStore, self).__init__()
-        self.source_store = store
-        # This clones the source store type with a locally bound
-        # transport. FIXME: it assumes a constructor is == cloning.
-        # clonable store - it might be nicer to actually have a clone()
-        # or something. RBC 20051003
-        self.cache_store = store.__class__(LocalTransport(cache_dir))
-
-    def get(self, id):
-        mutter("Cache add %s", id)
-        if id not in self.cache_store:
-            self.cache_store.add(self.source_store.get(id), id)
-        return self.cache_store.get(id)
-
-    def has_id(self, fileid, suffix=None):
-        """See Store.has_id."""
-        if self.cache_store.has_id(fileid, suffix):
-            return True
-        if self.source_store.has_id(fileid, suffix):
-            # We could asynchronously copy at this time
-            return True
-        return False
-
-
 def copy_all(store_from, store_to):
     """Copy all ids from one store to another."""
     # TODO: Optional progress indicator

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2005-11-10 02:11:13 +0000
+++ bzrlib/store/text.py	2005-12-15 21:11:09 +0000
@@ -65,15 +65,15 @@
 
     def _try_put(self, fn, f):
         try:
-            self._transport.put(fn, f)
+            self._transport.put(fn, f, mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
             try:
-                self._transport.mkdir(os.path.dirname(fn))
+                self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
             except FileExists:
                 pass
-            self._transport.put(fn, f)
+            self._transport.put(fn, f, mode=self._file_mode)
 
     def _get(self, fn):
         if fn.endswith('.gz'):
@@ -95,15 +95,17 @@
             raise KeyError(fileid + '-' + str(suffix))
 
         try:
-            result = other._transport.copy_to([path], self._transport, pb=pb)
+            result = other._transport.copy_to([path], self._transport, pb=pb,
+                                              mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
                 raise
             try:
-                self._transport.mkdir(hash_prefix(fileid)[:-1])
+                self._transport.mkdir(hash_prefix(fileid)[:-1], mode=self._dir_mode)
             except FileExists:
                 pass
-            result = other._transport.copy_to([path], self._transport, pb=pb)
+            result = other._transport.copy_to([path], self._transport, pb=pb,
+                                              mode=self._file_mode)
 
         if result != 1:
             raise BzrError('Unable to copy file: %r' % (path,))

=== modified file 'bzrlib/store/weave.py'
--- bzrlib/store/weave.py	2005-12-08 07:35:29 +0000
+++ bzrlib/store/weave.py	2005-12-19 20:31:01 +0000
@@ -40,9 +40,11 @@
     """
     FILE_SUFFIX = '.weave'
 
-    def __init__(self, transport, prefixed=False, precious=False):
-        self._transport = transport
-        self._prefixed = prefixed
+    def __init__(self, transport, prefixed=False, precious=False,
+                 dir_mode=None, file_mode=None):
+        super(WeaveStore, self).__init__(transport,
+                dir_mode=dir_mode, file_mode=file_mode,
+                prefixed=prefixed, compressed=False)
         self._precious = precious
 
     def filename(self, file_id):
@@ -67,10 +69,10 @@
     def _put(self, file_id, f):
         if self._prefixed:
             try:
-                self._transport.mkdir(hash_prefix(file_id))
+                self._transport.mkdir(hash_prefix(file_id), mode=self._dir_mode)
             except FileExists:
                 pass
-        return self._transport.put(self.filename(file_id), f)
+        return self._transport.put(self.filename(file_id), f, mode=self._file_mode)
 
     def get_weave(self, file_id, transaction):
         weave = transaction.map.find_weave(file_id)
@@ -80,6 +82,31 @@
         w = read_weave(self._get(file_id))
         transaction.map.add_weave(file_id, w)
         transaction.register_clean(w, precious=self._precious)
+        # TODO: jam 20051219 This should check if there is a prelude
+        #       which is already cached, and if so, should remove it
+        #       But transaction doesn't seem to have a 'remove'
+        #       One workaround would be to re-add the object with
+        #       the PRELUDE marker.
+        return w
+
+    def get_weave_prelude(self, file_id, transaction):
+        weave_id = file_id
+        weave = transaction.map.find_weave(weave_id)
+        if weave:
+            mutter("cache hit in %s for %s", self, weave_id)
+            return weave
+        # We want transactions to also cache preludes if that
+        # is all that we are loading. So we need a unique
+        # identifier, so that someone who wants the whole text
+        # won't get just the prelude
+        weave_id = 'PRELUDE-' + file_id
+        weave = transaction.map.find_weave(weave_id)
+        if weave:
+            mutter("cache hit in %s for %s", self, weave_id)
+            return weave
+        w = read_weave(self._get(file_id), prelude=True)
+        transaction.map.add_weave(weave_id, w)
+        transaction.register_clean(w, precious=self._precious)
         return w
 
     def get_lines(self, file_id, rev_id, transaction):
@@ -92,14 +119,15 @@
     def get_weave_prelude_or_empty(self, file_id, transaction):
         """cheap version that reads the prelude but not the lines
         """
-        weave = transaction.map.find_weave(file_id)
-        if weave:
-            mutter("cache hit in %s for %s", self, file_id)
-            return weave
         try:
-            return read_weave(self._get(file_id),prelude=True)
+            return self.get_weave_prelude(file_id, transaction)
         except NoSuchFile:
-            return Weave(weave_name=file_id)
+            # We can cache here, because we know that there
+            # is no complete object, since we got NoSuchFile
+            weave = Weave(weave_name=file_id)
+            transaction.map.add_weave(file_id, weave)
+            transaction.register_clean(weave, precious=self._precious)
+            return weave
 
     def get_weave_or_empty(self, file_id, transaction):
         """Return a weave, or an empty one if it doesn't exist.""" 

=== modified file 'bzrlib/tests/HTTPTestUtil.py'
--- bzrlib/tests/HTTPTestUtil.py	2005-11-25 06:28:56 +0000
+++ bzrlib/tests/HTTPTestUtil.py	2005-12-01 19:42:25 +0000
@@ -16,15 +16,12 @@
 
 import BaseHTTPServer, SimpleHTTPServer, socket, errno, time
 from bzrlib.tests import TestCaseInTempDir
+from bzrlib.osutils import relpath
 
 
 class WebserverNotAvailable(Exception):
     pass
 
-class BadWebserverPath(ValueError):
-    def __str__(self):
-        return 'path %s is not in %s' % self.args
-
 class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
     def log_message(self, format, *args):
         self.server.test_case.log("webserver - %s - - [%s] %s",
@@ -116,14 +113,10 @@
     def get_remote_url(self, path):
         import os
 
-        path_parts = path.split(os.path.sep)
         if os.path.isabs(path):
-            if path_parts[:len(self._local_path_parts)] != \
-                   self._local_path_parts:
-                raise BadWebserverPath(path, self.test_dir)
-            remote_path = '/'.join(path_parts[len(self._local_path_parts):])
+            remote_path = relpath(self.test_dir, path)
         else:
-            remote_path = '/'.join(path_parts)
+            remote_path = path
 
         self._http_starting.acquire()
         self._http_starting.release()
@@ -132,7 +125,6 @@
     def setUp(self):
         TestCaseInTempDir.setUp(self)
         import threading, os
-        self._local_path_parts = self.test_dir.split(os.path.sep)
         self._http_starting = threading.Lock()
         self._http_starting.acquire()
         self._http_running = True

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2005-12-31 05:16:47 +0000
+++ bzrlib/tests/__init__.py	2006-01-03 23:51:45 +0000
@@ -34,6 +34,7 @@
 import bzrlib.inventory
 import bzrlib.merge3
 import bzrlib.osutils
+import bzrlib.osutils as osutils
 import bzrlib.plugin
 import bzrlib.store
 import bzrlib.trace
@@ -97,7 +98,7 @@
         # at the end
         SHOW_DESCRIPTIONS = False
         if self.showAll:
-            width = bzrlib.osutils.terminal_width()
+            width = osutils.terminal_width()
             name_width = width - 15
             what = None
             if SHOW_DESCRIPTIONS:
@@ -257,6 +258,14 @@
             return
         raise AssertionError("texts not equal:\n" + 
                              self._ndiff_strings(a, b))      
+        
+    def assertStartsWith(self, s, prefix):
+        if not s.startswith(prefix):
+            raise AssertionError('string %r does not start with %r' % (s, prefix))
+
+    def assertEndsWith(self, s, suffix):
+        if not s.endswith(prefix):
+            raise AssertionError('string %r does not end with %r' % (s, suffix))
 
     def assertContainsRe(self, haystack, needle_re):
         """Assert that a contains something matching a regular expression."""
@@ -524,11 +533,11 @@
                 else:
                     raise
             # successfully created
-            TestCaseInTempDir.TEST_ROOT = os.path.abspath(root)
+            TestCaseInTempDir.TEST_ROOT = osutils.abspath(root)
             break
         # make a fake bzr directory there to prevent any tests propagating
         # up onto the source directory's real branch
-        os.mkdir(os.path.join(TestCaseInTempDir.TEST_ROOT, '.bzr'))
+        os.mkdir(osutils.pathjoin(TestCaseInTempDir.TEST_ROOT, '.bzr'))
 
     def setUp(self):
         super(TestCaseInTempDir, self).setUp()
@@ -536,10 +545,11 @@
         _currentdir = os.getcwdu()
         short_id = self.id().replace('bzrlib.tests.', '') \
                    .replace('__main__.', '')
-        self.test_dir = os.path.join(self.TEST_ROOT, short_id)
+        self.test_dir = osutils.pathjoin(self.TEST_ROOT, short_id)
         os.mkdir(self.test_dir)
         os.chdir(self.test_dir)
         os.environ['HOME'] = self.test_dir
+        os.environ['APPDATA'] = self.test_dir
         def _leaveDirectory():
             os.chdir(_currentdir)
         self.addCleanup(_leaveDirectory)
@@ -576,13 +586,17 @@
 
     def failUnlessExists(self, path):
         """Fail unless path, which may be abs or relative, exists."""
-        self.failUnless(bzrlib.osutils.lexists(path))
+        self.failUnless(osutils.lexists(path))
+
+    def failIfExists(self, path):
+        """Fail if path, which may be abs or relative, exists."""
+        self.failIf(osutils.lexists(path))
         
     def assertFileEqual(self, content, path):
         """Fail if path does not contain 'content'."""
-        self.failUnless(bzrlib.osutils.lexists(path))
+        self.failUnless(osutils.lexists(path))
         self.assertEqualDiff(content, open(path, 'r').read())
-        
+
 
 def filter_suite_by_re(suite, pattern):
     result = TestSuite()
@@ -636,6 +650,7 @@
                    'bzrlib.tests.test_annotate',
                    'bzrlib.tests.test_api',
                    'bzrlib.tests.test_bad_files',
+                   'bzrlib.tests.test_basis_inventory',
                    'bzrlib.tests.test_bound_sftp',
                    'bzrlib.tests.test_branch',
                    'bzrlib.tests.test_command',
@@ -659,11 +674,12 @@
                    'bzrlib.tests.test_msgeditor',
                    'bzrlib.tests.test_nonascii',
                    'bzrlib.tests.test_options',
+                   'bzrlib.tests.test_osutils',
                    'bzrlib.tests.test_parent',
+                   'bzrlib.tests.test_permissions',
                    'bzrlib.tests.test_plugins',
                    'bzrlib.tests.test_remove',
                    'bzrlib.tests.test_revision',
-                   'bzrlib.tests.test_revision_info',
                    'bzrlib.tests.test_revisionnamespaces',
                    'bzrlib.tests.test_revprops',
                    'bzrlib.tests.test_reweave',
@@ -671,7 +687,7 @@
                    'bzrlib.tests.test_sampler',
                    'bzrlib.tests.test_selftest',
                    'bzrlib.tests.test_setup',
-                   'bzrlib.tests.test_sftp',
+                   'bzrlib.tests.test_sftp_transport',
                    'bzrlib.tests.test_smart_add',
                    'bzrlib.tests.test_source',
                    'bzrlib.tests.test_status',
@@ -690,7 +706,9 @@
                    'bzrlib.tests.test_xml',
                    ]
 
-    print '%10s: %s' % ('bzr', os.path.realpath(sys.argv[0]))
+    TestCase.BZRPATH = osutils.pathjoin(
+            osutils.realpath(osutils.dirname(bzrlib.__path__[0])), 'bzr')
+    print '%10s: %s' % ('bzr', osutils.realpath(sys.argv[0]))
     print '%10s: %s' % ('bzrlib', bzrlib.__path__[0])
     print
     suite = TestSuite()

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2005-12-30 22:46:21 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-01-03 23:51:45 +0000
@@ -30,11 +30,13 @@
     testmod_names = [
                      'bzrlib.tests.blackbox.test_bound_branches',
                      'bzrlib.tests.blackbox.test_cat',
-                     'bzrlib.tests.blackbox.test_too_much',
+                     'bzrlib.tests.blackbox.test_export',
+                     'bzrlib.tests.blackbox.test_missing',
                      'bzrlib.tests.blackbox.test_pull',
                      'bzrlib.tests.blackbox.test_revno',
+                     'bzrlib.tests.blackbox.test_revision_info',
+                     'bzrlib.tests.blackbox.test_too_much',
                      'bzrlib.tests.blackbox.test_versioning',
-                     'bzrlib.tests.blackbox.test_missing'
                      ]
     return TestLoader().loadTestsFromNames(testmod_names)
 

=== added file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_export.py	2005-12-30 03:25:20 +0000
@@ -0,0 +1,74 @@
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr export.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+import tarfile, zipfile
+
+class TestExport(ExternalBase):
+
+    def test_tar_export(self):
+
+        os.mkdir('tar')
+        os.chdir('tar')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export test.tar.gz')
+        ball = tarfile.open('test.tar.gz')
+        for m in ball.getnames():
+            #print m
+            self.failIf(os.path.basename(m) == '.bzrignore', 'tar export contains .bzrignore')
+
+    def test_zip_export(self):
+
+        os.mkdir('zip')
+        os.chdir('zip')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export test.zip')
+        ball = zipfile.ZipFile('test.zip')
+        for m in ball.namelist():
+            #print m
+            self.failIf(os.path.basename(m) == '.bzrignore', 'zip export contains .bzrignore')
+
+    def test_dir_export(self):
+
+        os.mkdir('dir')
+        os.chdir('dir')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export direxport')
+
+        files = sorted(os.listdir('direxport'))
+        self.assertEqual(['a'], files, 'dir contains .bzrignore')

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2005-12-01 15:49:10 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2005-12-01 21:49:24 +0000
@@ -20,6 +20,7 @@
 """
 
 import os
+import sys
 
 from bzrlib.branch import Branch
 from bzrlib.tests.blackbox import ExternalBase
@@ -45,9 +46,18 @@
         self.runbzr('missing', retcode=3)
         self.runbzr('missing .')
         self.runbzr('missing')
-        self.runbzr('pull')
+        if sys.platform not in ('win32', 'cygwin'):
+            # This is equivalent to doing "bzr pull ."
+            # Which means that bzr creates 2 branches grabbing
+            # the same location, and tries to pull.
+            # However, 2 branches mean 2 locks on the same file
+            # which ultimately implies a deadlock.
+            # (non windows platforms allow multiple locks on the
+            # same file by the same calling process)
+            self.runbzr('pull')
         self.runbzr('pull /', retcode=3)
-        self.runbzr('pull')
+        if sys.platform not in ('win32', 'cygwin'):
+            self.runbzr('pull')
 
         os.chdir('..')
         self.runbzr('branch a b')

=== renamed file 'bzrlib/tests/test_revision_info.py' => 'bzrlib/tests/blackbox/test_revision_info.py'
=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2005-12-13 20:13:15 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2005-12-18 04:08:46 +0000
@@ -43,7 +43,7 @@
 from bzrlib.branch import Branch
 from bzrlib.clone import copy_branch
 from bzrlib.errors import BzrCommandError
-from bzrlib.osutils import has_symlinks
+from bzrlib.osutils import has_symlinks, pathjoin
 from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
 from bzrlib.tests.blackbox import ExternalBase
 
@@ -215,17 +215,20 @@
         os.rmdir('revertdir')
         self.runbzr('revert')
 
-        os.symlink('/unlikely/to/exist', 'symlink')
-        self.runbzr('add symlink')
-        self.runbzr('commit -m f')
-        os.unlink('symlink')
-        self.runbzr('revert')
-        self.failUnlessExists('symlink')
-        os.unlink('symlink')
-        os.symlink('a-different-path', 'symlink')
-        self.runbzr('revert')
-        self.assertEqual('/unlikely/to/exist',
-                         os.readlink('symlink'))
+        if has_symlinks():
+            os.symlink('/unlikely/to/exist', 'symlink')
+            self.runbzr('add symlink')
+            self.runbzr('commit -m f')
+            os.unlink('symlink')
+            self.runbzr('revert')
+            self.failUnlessExists('symlink')
+            os.unlink('symlink')
+            os.symlink('a-different-path', 'symlink')
+            self.runbzr('revert')
+            self.assertEqual('/unlikely/to/exist',
+                             os.readlink('symlink'))
+        else:
+            self.log("skipping revert symlink tests")
         
         file('hello', 'wt').write('xyz')
         self.runbzr('commit -m xyz hello')
@@ -674,7 +677,7 @@
         self.assertEquals(['If you wish to add some of these files, please'\
                            ' add them by name.',
                            'added dir',
-                           'added dir'+os.sep+'sub.txt',
+                           'added dir/sub.txt',
                            'added top.txt',
                            'ignored 1 file(s) matching "CVS"'],
                           results)
@@ -728,7 +731,7 @@
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
         self.run_bzr('add', '--no-recurse', 'inertiatic')
-        self.assertEquals(self.capture('unknowns'), 'inertiatic'+os.sep+'esp\n')
+        self.assertEquals(self.capture('unknowns'), 'inertiatic/esp\n')
         self.run_bzr('add', 'inertiatic/esp')
         self.assertEquals(self.capture('unknowns'), '')
 
@@ -1007,7 +1010,7 @@
         runbzr('init')
 
         self.assertEquals(capture('root').rstrip(),
-                          os.path.join(self.test_dir, 'branch1'))
+                          pathjoin(self.test_dir, 'branch1'))
 
         progress("status of new file")
 
@@ -1083,7 +1086,7 @@
         runbzr("rename sub1 sub2")
         runbzr("move hello.txt sub2")
         self.assertEqual(capture("relpath sub2/hello.txt"),
-                         os.path.join("sub2", "hello.txt\n"))
+                         pathjoin("sub2", "hello.txt\n"))
 
         self.assert_(exists("sub2"))
         self.assert_(exists("sub2/hello.txt"))
@@ -1105,15 +1108,15 @@
 
         chdir('sub1/sub2')
         self.assertEquals(capture('root')[:-1],
-                          os.path.join(self.test_dir, 'branch1'))
+                          pathjoin(self.test_dir, 'branch1'))
         runbzr('move ../hello.txt .')
         self.assert_(exists('./hello.txt'))
         self.assertEquals(capture('relpath hello.txt'),
-                          os.path.join('sub1', 'sub2', 'hello.txt') + '\n')
-        self.assertEquals(capture('relpath ../../sub1/sub2/hello.txt'), os.path.join('sub1', 'sub2', 'hello.txt\n'))
+                          pathjoin('sub1', 'sub2', 'hello.txt') + '\n')
+        self.assertEquals(capture('relpath ../../sub1/sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
         runbzr(['commit', '-m', 'move to parent directory'])
         chdir('..')
-        self.assertEquals(capture('relpath sub2/hello.txt'), os.path.join('sub1', 'sub2', 'hello.txt\n'))
+        self.assertEquals(capture('relpath sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
 
         runbzr('move sub2/hello.txt .')
         self.assert_(exists('hello.txt'))

=== modified file 'bzrlib/tests/blackbox/test_versioning.py'
--- bzrlib/tests/blackbox/test_versioning.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/blackbox/test_versioning.py	2005-12-01 17:11:25 +0000
@@ -25,6 +25,7 @@
 from bzrlib.tests import BzrTestBase, TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.trace import mutter
+from bzrlib.osutils import pathjoin
 
 
 class TestVersioning(TestCaseInTempDir):
@@ -73,7 +74,7 @@
 
         self.assertEquals(len(delta.added), 2)
         self.assertEquals(delta.added[0][0], 'dir')
-        self.assertEquals(delta.added[1][0], os.path.join('dir','subdir'))
+        self.assertEquals(delta.added[1][0], pathjoin('dir','subdir'))
         self.failIf(delta.modified)
 
     def test_mkdir_w_nested_trees(self):

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2005-11-20 17:00:43 +0000
+++ bzrlib/tests/stub_sftp.py	2005-12-18 04:08:46 +0000
@@ -22,6 +22,8 @@
 import os
 from paramiko import ServerInterface, SFTPServerInterface, SFTPServer, SFTPAttributes, \
     SFTPHandle, SFTP_OK, AUTH_SUCCESSFUL, OPEN_SUCCEEDED
+from bzrlib.osutils import pathjoin
+from bzrlib.trace import mutter
 
 
 class StubServer (ServerInterface):
@@ -49,6 +51,7 @@
     def chattr(self, attr):
         # python doesn't have equivalents to fchown or fchmod, so we have to
         # use the stored filename
+        mutter('Changing permissions on %s to %s', self.filename, attr)
         try:
             SFTPServer.set_file_attr(self.filename, attr)
         except OSError, e:
@@ -63,13 +66,20 @@
     def _realpath(self, path):
         return self.root + self.canonicalize(path)
 
+    def chattr(self, path, attr):
+        try:
+            SFTPServer.set_file_attr(path, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
     def list_folder(self, path):
         path = self._realpath(path)
         try:
             out = [ ]
             flist = os.listdir(path)
             for fname in flist:
-                attr = SFTPAttributes.from_stat(os.stat(os.path.join(path, fname)))
+                attr = SFTPAttributes.from_stat(os.stat(pathjoin(path, fname)))
                 attr.filename = fname
                 out.append(attr)
             return out
@@ -93,18 +103,24 @@
     def open(self, path, flags, attr):
         path = self._realpath(path)
         try:
-            fd = os.open(path, flags)
+            if hasattr(os, 'O_BINARY'):
+                flags |= os.O_BINARY
+            if (attr is not None) and hasattr(attr, 'st_mode'):
+                fd = os.open(path, flags, attr.st_mode)
+            else:
+                fd = os.open(path, flags)
         except OSError, e:
             return SFTPServer.convert_errno(e.errno)
         if (flags & os.O_CREAT) and (attr is not None):
+            attr._flags &= ~attr.FLAG_PERMISSIONS
             SFTPServer.set_file_attr(path, attr)
         if flags & os.O_WRONLY:
-            fstr = 'w'
+            fstr = 'wb'
         elif flags & os.O_RDWR:
-            fstr = 'r+'
+            fstr = 'rb+'
         else:
             # O_RDONLY (== 0)
-            fstr = 'r'
+            fstr = 'rb'
         try:
             f = os.fdopen(fd, fstr)
         except OSError, e:
@@ -135,8 +151,12 @@
     def mkdir(self, path, attr):
         path = self._realpath(path)
         try:
-            os.mkdir(path)
+            if attr is not None and hasattr(attr, 'st_mode'):
+                os.mkdir(path, attr.st_mode)
+            else:
+                os.mkdir(path)
             if attr is not None:
+                attr._flags &= ~attr.FLAG_PERMISSIONS
                 SFTPServer.set_file_attr(path, attr)
         except OSError, e:
             return SFTPServer.convert_errno(e.errno)

=== added file 'bzrlib/tests/test_basis_inventory.py'
--- bzrlib/tests/test_basis_inventory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_basis_inventory.py	2006-01-02 22:37:32 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.xml5 import serializer_v5
+
+
+class TestBasisInventory(TestCaseInTempDir):
+
+    def test_create(self):
+        # Make sure the basis file is created by a commit
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        open('a', 'wb').write('a\n')
+        t.add('a')
+        t.commit('a', rev_id='r1')
+
+        self.failUnlessExists('.bzr/basis-inventory.r1')
+
+        basis_inv_txt = t.read_basis_inventory('r1')
+        basis_inv = serializer_v5.read_inventory_from_string(basis_inv_txt)
+        #self.assertEquals('r1', basis_inv.revision_id)
+        
+        store_inv = b.get_inventory('r1')
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+        open('b', 'wb').write('b\n')
+        t.add('b')
+        t.commit('b', rev_id='r2')
+
+        self.failIfExists('.bzr/basis-inventory.r1')
+        self.failUnlessExists('.bzr/basis-inventory.r2')
+
+        basis_inv_txt = t.read_basis_inventory('r2')
+        basis_inv = serializer_v5.read_inventory_from_string(basis_inv_txt)
+        store_inv = b.get_inventory('r2')
+
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+

=== modified file 'bzrlib/tests/test_bound_sftp.py'
--- bzrlib/tests/test_bound_sftp.py	2005-12-31 15:19:21 +0000
+++ bzrlib/tests/test_bound_sftp.py	2006-01-03 23:51:45 +0000
@@ -20,7 +20,7 @@
 
 import os
 
-from bzrlib.tests.test_sftp import TestCaseWithSFTPServer, paramiko_loaded
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer, paramiko_loaded
 import bzrlib.errors as errors
 from bzrlib.branch import Branch
 from bzrlib.clone import copy_branch

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_branch.py	2005-12-01 19:57:43 +0000
@@ -15,6 +15,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import sys
 
 from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.clone import copy_branch
@@ -22,6 +23,7 @@
 import bzrlib.errors as errors
 from bzrlib.errors import NoSuchRevision, UnlistableBranch, NotBranchError
 import bzrlib.gpg
+from bzrlib.osutils import getcwd
 from bzrlib.tests import TestCase, TestCaseInTempDir
 from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
 from bzrlib.trace import mutter
@@ -49,7 +51,7 @@
         os.mkdir('b2')
         b1 = Branch.initialize('b1')
         b2 = Branch.initialize('b2')
-        file(os.sep.join(['b1', 'foo']), 'w').write('hello')
+        file('b1/foo', 'w').write('hello')
         b1.working_tree().add(['foo'], ['foo-id'])
         b1.working_tree().commit('lala!', rev_id='revision-1', allow_pointless=False)
 
@@ -374,17 +376,24 @@
         self.assertEqual(None, self.branch.get_push_location())
 
     def test_get_push_location_exact(self):
-        self.build_tree(['.bazaar/'])
-        print >> open('.bazaar/branches.conf', 'wt'), ("[%s]\n"
-                                                       "push_location=foo" %
-                                                       os.getcwdu())
+        from bzrlib.config import (branches_config_filename,
+                                   ensure_config_dir_exists)
+        ensure_config_dir_exists()
+        fn = branches_config_filename()
+        print >> open(fn, 'wt'), ("[%s]\n"
+                                  "push_location=foo" %
+                                  getcwd())
         self.assertEqual("foo", self.branch.get_push_location())
 
     def test_set_push_location(self):
+        from bzrlib.config import (branches_config_filename,
+                                   ensure_config_dir_exists)
+        ensure_config_dir_exists()
+        fn = branches_config_filename()
         self.branch.set_push_location('foo')
         self.assertFileEqual("[%s]\n"
-                             "push_location = foo" % os.getcwdu(),
-                             '.bazaar/branches.conf')
+                             "push_location = foo" % getcwd(),
+                             fn)
 
     # TODO RBC 20051029 test getting a push location from a branch in a 
     # recursive section - that is, it appends the branch name.

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_commit.py	2005-12-01 17:36:29 +0000
@@ -201,8 +201,7 @@
         self.check_inventory_shape(b.get_revision_inventory(r3),
                                    ['a', 'a/hello', 'a/b'])
 
-        b.working_tree().move([os.sep.join(['a', 'hello'])],
-               os.sep.join(['a', 'b']))
+        b.working_tree().move(['a/hello'], 'a/b')
         r4 = 'test@rev-4'
         b.working_tree().commit('four', rev_id=r4, allow_pointless=False)
         self.check_inventory_shape(b.working_tree().read_working_inventory(),

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2005-12-01 21:31:42 +0000
+++ bzrlib/tests/test_config.py	2005-12-30 22:03:42 +0000
@@ -195,14 +195,14 @@
     def test_config_dir(self):
         if sys.platform == 'win32':
             self.assertEqual(config.config_dir(), 
-                r'C:\Documents and Settings\bogus\Application Data\bazaar\2.0')
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0')
         else:
             self.assertEqual(config.config_dir(), '/home/bogus/.bazaar')
 
     def test_config_filename(self):
         if sys.platform == 'win32':
             self.assertEqual(config.config_filename(), 
-                r'C:\Documents and Settings\bogus\Application Data\bazaar\2.0\bazaar.conf')
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/bazaar.conf')
         else:
             self.assertEqual(config.config_filename(),
                              '/home/bogus/.bazaar/bazaar.conf')
@@ -210,7 +210,7 @@
     def test_branches_config_filename(self):
         if sys.platform == 'win32':
             self.assertEqual(config.branches_config_filename(), 
-                r'C:\Documents and Settings\bogus\Application Data\bazaar\2.0\branches.conf')
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/branches.conf')
         else:
             self.assertEqual(config.branches_config_filename(),
                              '/home/bogus/.bazaar/branches.conf')
@@ -507,14 +507,24 @@
         self.my_config._get_global_config()._get_parser(global_file)
 
     def test_set_user_setting_sets_and_saves(self):
-        # TODO RBC 20051029 test hat mkdir ~/.bazaar is called ..
         self.get_location_config('/a/c')
         record = InstrumentedConfigObj("foo")
         self.my_config._parser = record
-        print ("test_set_user_setting_sets_and_saves broken: creates .bazaar "
-               "in the top-level directory, not inside the test directory")
-        return
-        self.my_config.set_user_option('foo', 'bar')
+
+        real_mkdir = os.mkdir
+        self.created = False
+        def checked_mkdir(path, mode=0777):
+            self.log('making directory: %s', path)
+            real_mkdir(path, mode)
+            self.created = True
+
+        os.mkdir = checked_mkdir
+        try:
+            self.my_config.set_user_option('foo', 'bar')
+        finally:
+            os.mkdir = real_mkdir
+
+        self.failUnless(self.created, 'Failed to create ~/.bazaar')
         self.assertEqual([('__contains__', '/a/c'),
                           ('__contains__', '/a/c/'),
                           ('__setitem__', '/a/c', {}),

=== modified file 'bzrlib/tests/test_gpg.py'
--- bzrlib/tests/test_gpg.py	2005-11-23 15:44:24 +0000
+++ bzrlib/tests/test_gpg.py	2005-12-16 02:25:56 +0000
@@ -47,9 +47,22 @@
     def test_returns_output(self):
         # This test needs a 'cat' command or similar to work.
         my_gpg = gpg.GPGStrategy(FakeConfig())
-        my_gpg._command_line = lambda:["cat", "-"]
-        self.assertEqual("some content\nwith newlines\n",
-                         my_gpg.sign("some content\nwith newlines\n"))
+        content = "some content\nwith newlines\n"
+
+        if sys.platform == 'win32':
+            # Windows doesn't come with cat, and we don't require it
+            # so lets try using python instead.
+            # But stupid windows and line-ending conversions. 
+            # It is too much work to make sys.stdout be in binary mode.
+            # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/65443
+            my_gpg._command_line = lambda:[sys.executable, '-c',
+                    'import sys; sys.stdout.write(sys.stdin.read())']
+            new_content = content.replace('\n', '\r\n')
+
+            self.assertEqual(new_content, my_gpg.sign(content))
+        else:
+            my_gpg._command_line = lambda:['cat', '-']
+            self.assertEqual(content, my_gpg.sign(content))
 
 
 class TestDisabled(TestCase):

=== modified file 'bzrlib/tests/test_hashcache.py'
--- bzrlib/tests/test_hashcache.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_hashcache.py	2005-12-26 05:28:38 +0000
@@ -61,21 +61,21 @@
         # check we hit without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        ##self.assertEquals(hc.miss_count, 1)
-        ##self.assertEquals(hc.hit_count, 1)
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 1)
 
         # check again without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        ##self.assertEquals(hc.miss_count, 1)
-        ##self.assertEquals(hc.hit_count, 2)
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 2)
 
         # write new file and make sure it is seen
         file('foo', 'wb').write('goodbye')
         pause()
         self.assertEquals(hc.get_sha1('foo'),
                           '3c8ec4874488f6090a157b014ce3397ca8e06d4f')
-        ##self.assertEquals(hc.miss_count, 2)
+        self.assertEquals(hc.miss_count, 2)
 
         # quickly write new file of same size and make sure it is seen
         # this may rely on detection of timestamps that are too close
@@ -101,9 +101,9 @@
 
         # should now be safe to cache it if we reread them
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        ##self.assertEquals(len(hc._cache), 1)
+        self.assertEquals(len(hc._cache), 1)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-        ##self.assertEquals(len(hc._cache), 2)
+        self.assertEquals(len(hc._cache), 2)
 
         # write out, read back in and check that we don't need to
         # re-read any files
@@ -113,8 +113,8 @@
         hc = HashCache(u'.')
         hc.read()
 
-        ##self.assertEquals(len(hc._cache), 2)
+        self.assertEquals(len(hc._cache), 2)
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        ##self.assertEquals(hc.hit_count, 1)
-        ##self.assertEquals(hc.miss_count, 0)
+        self.assertEquals(hc.hit_count, 1)
+        self.assertEquals(hc.miss_count, 0)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_inv.py	2005-12-08 18:32:43 +0000
@@ -23,7 +23,7 @@
 from bzrlib.diff import internal_diff
 from bzrlib.inventory import Inventory, ROOT_ID
 import bzrlib.inventory as inventory
-from bzrlib.osutils import has_symlinks, rename
+from bzrlib.osutils import has_symlinks, rename, pathjoin
 from bzrlib.tests import TestCase, TestCaseInTempDir
 
 
@@ -32,7 +32,7 @@
     def test_is_within(self):
         from bzrlib.osutils import is_inside_any
 
-        SRC_FOO_C = os.path.join('src', 'foo.c')
+        SRC_FOO_C = pathjoin('src', 'foo.c')
         for dirs, fn in [(['src', 'doc'], SRC_FOO_C),
                          (['src'], SRC_FOO_C),
                          (['src'], 'src'),
@@ -374,3 +374,119 @@
                          self.get_previous_heads([self.inv_D, self.inv_B]))
 
     # TODO: test two inventories with the same file revision 
+
+
+class TestExecutable(TestCaseInTempDir):
+
+    def test_stays_executable(self):
+        basic_inv = """<inventory format="5">
+<file file_id="a-20051208024829-849e76f7968d7a86" name="a" executable="yes" />
+<file file_id="b-20051208024829-849e76f7968d7a86" name="b" />
+</inventory>
+"""
+        os.mkdir('b1')
+        b = Branch.initialize('b1')
+        open('b1/a', 'wb').write('a test\n')
+        open('b1/b', 'wb').write('b test\n')
+        os.chmod('b1/a', 0755)
+        os.chmod('b1/b', 0644)
+        # Manually writing the inventory, to ensure that
+        # the executable="yes" entry is set for 'a' and not for 'b'
+        open('b1/.bzr/inventory', 'wb').write(basic_inv)
+
+        a_id = "a-20051208024829-849e76f7968d7a86"
+        b_id = "b-20051208024829-849e76f7968d7a86"
+        t = b.working_tree()
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t.inventory.iter_entries()])
+
+        self.failUnless(t.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t.is_executable(b_id), "'b' gained an execute bit")
+
+        t.commit('adding a,b', rev_id='r1')
+
+        rev_tree = b.revision_tree('r1')
+        self.failUnless(rev_tree.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(rev_tree.is_executable(b_id), "'b' gained an execute bit")
+
+        self.failUnless(rev_tree.inventory[a_id].executable)
+        self.failIf(rev_tree.inventory[b_id].executable)
+
+        # Make sure the entries are gone
+        os.remove('b1/a')
+        os.remove('b1/b')
+        self.failIf(t.has_id(a_id))
+        self.failIf(t.has_filename('a'))
+        self.failIf(t.has_id(b_id))
+        self.failIf(t.has_filename('b'))
+
+        # Make sure that revert is able to bring them back,
+        # and sets 'a' back to being executable
+
+        t.revert(['b1/a', 'b1/b'], rev_tree, backups=False)
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t.inventory.iter_entries()])
+
+        self.failUnless(t.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t.is_executable(b_id), "'b' gained an execute bit")
+
+        # Now remove them again, and make sure that after a
+        # commit, they are still marked correctly
+        os.remove('b1/a')
+        os.remove('b1/b')
+        t.commit('removed', rev_id='r2')
+
+        self.assertEqual([], [cn for cn,ie in t.inventory.iter_entries()])
+        self.failIf(t.has_id(a_id))
+        self.failIf(t.has_filename('a'))
+        self.failIf(t.has_id(b_id))
+        self.failIf(t.has_filename('b'))
+
+        # Now revert back to the previous commit
+        t.revert([], rev_tree, backups=False)
+        # TODO: FIXME: For some reason, after revert, the tree does not 
+        # regenerate its working inventory, so we have to manually delete
+        # the working tree, and create a new one
+        # This seems to happen any time you do a merge operation on the
+        # working tree
+        del t
+        t = b.working_tree()
+
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t.inventory.iter_entries()])
+
+        self.failUnless(t.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t.is_executable(b_id), "'b' gained an execute bit")
+
+        # Now make sure that 'bzr branch' also preserves the
+        # executable bit
+        # TODO: Maybe this should be a blackbox test
+        from bzrlib.clone import copy_branch
+        copy_branch(b, 'b2', revision='r1')
+        b2 = Branch.open('b2')
+        self.assertEquals('r1', b2.last_revision())
+        t2 = b2.working_tree()
+
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t2.inventory.iter_entries()])
+        self.failUnless(t2.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t2.is_executable(b_id), "'b' gained an execute bit")
+
+        # Make sure pull will delete the files
+        t2.pull(b)
+        self.assertEquals('r2', b2.last_revision())
+        # FIXME: Same thing here, t2 needs to be recreated
+        del t2
+        t2 = b2.working_tree()
+        self.assertEqual([], [cn for cn,ie in t2.inventory.iter_entries()])
+
+        # Now commit the changes on the first branch
+        # so that the second branch can pull the changes
+        # and make sure that the executable bit has been copied
+        t.commit('resurrected', rev_id='r3')
+
+        t2.pull(b)
+        # FIXME: And here
+        del t2
+        t2 = b2.working_tree()
+        self.assertEquals('r3', b2.last_revision())
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t2.inventory.iter_entries()])
+
+        self.failUnless(t2.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t2.is_executable(b_id), "'b' gained an execute bit")

=== modified file 'bzrlib/tests/test_log.py'
--- bzrlib/tests/test_log.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_log.py	2005-12-01 17:36:29 +0000
@@ -57,8 +57,8 @@
             expected = kw.get(n, [])
 
             # tests are written with unix paths; fix them up for windows
-            if os.sep != '/':
-                expected = [x.replace('/', os.sep) for x in expected]
+            #if os.sep != '/':
+            #    expected = [x.replace('/', os.sep) for x in expected]
 
             # strip out only the path components
             got = [x[0] for x in getattr(delta, n)]

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_merge.py	2005-12-01 17:11:25 +0000
@@ -7,6 +7,7 @@
 from bzrlib.errors import UnrelatedBranches, NoCommits, BzrCommandError
 from bzrlib.revision import common_ancestor
 from bzrlib.fetch import fetch
+from bzrlib.osutils import pathjoin
 
 
 class TestMerge(TestCaseInTempDir):
@@ -70,11 +71,11 @@
         tree = b.working_tree()
         os.mkdir('dirname1')
         tree.add('dirname1')
-        filename = os.path.join('dirname1', 'name1')
+        filename = pathjoin('dirname1', 'name1')
         file(filename, 'wb').write('Hello')
         tree.add(filename)
         tree.commit(message="hello")
-        filename2 = os.path.join('dirname1', 'name2')
+        filename2 = pathjoin('dirname1', 'name2')
         tree.rename_one(filename, filename2)
         tree.rename_one('dirname1', 'dirname2')
         transform_tree(tree, b.basis_tree())

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2005-12-09 07:12:46 +0000
+++ bzrlib/tests/test_merge_core.py	2005-12-22 23:32:43 +0000
@@ -1,7 +1,5 @@
 import os
 import shutil
-import tempfile
-import unittest
 import stat
 import sys
 
@@ -11,7 +9,7 @@
                            WorkingTreeNotRevision, BzrCommandError)
 from bzrlib.inventory import RootEntry
 import bzrlib.inventory as inventory
-from bzrlib.osutils import file_kind, rename, sha_file
+from bzrlib.osutils import file_kind, rename, sha_file, pathjoin, mkdtemp
 from bzrlib import changeset
 from bzrlib.merge_core import (ApplyMerge3, make_merge_changeset,
                                BackupBeforeChange, ExecFlagMerge, WeaveMerge)
@@ -64,7 +62,7 @@
         self.inventory = FalseTree(self)
     
     def child_path(self, parent, name):
-        return os.path.join(self.inventory_dict[parent], name)
+        return pathjoin(self.inventory_dict[parent], name)
 
     def add_file(self, id, parent, name, contents, mode):
         path = self.child_path(parent, name)
@@ -94,7 +92,7 @@
         self.inventory_dict[id] = path
 
     def abs_path(self, path):
-        return os.path.join(self.dir, path)
+        return pathjoin(self.dir, path)
 
     def full_path(self, id):
         try:
@@ -123,7 +121,7 @@
         return self.full_path(id)
 
     def change_path(self, id, path):
-        old_path = os.path.join(self.dir, self.inventory_dict[id])
+        old_path = pathjoin(self.dir, self.inventory_dict[id])
         rename(old_path, self.abs_path(path))
         self.inventory_dict[id] = path
 
@@ -140,21 +138,23 @@
     def get_file_sha1(self, file_id):
         return sha_file(file(self.full_path(file_id), "rb"))
 
+
 class MergeBuilder(object):
     def __init__(self):
-        self.dir = tempfile.mkdtemp(prefix="BaZing")
-        self.base = MergeTree(os.path.join(self.dir, "base"))
-        self.this = MergeTree(os.path.join(self.dir, "this"))
-        self.other = MergeTree(os.path.join(self.dir, "other"))
+        self.dir = mkdtemp(prefix="BaZing")
+        self.base = MergeTree(pathjoin(self.dir, "base"))
+        self.this = MergeTree(pathjoin(self.dir, "this"))
+        self.other = MergeTree(pathjoin(self.dir, "other"))
         
         self.cset = changeset.Changeset()
         self.cset.add_entry(changeset.ChangesetEntry("0", 
                                                      changeset.NULL_ID, "./."))
+
     def get_cset_path(self, parent, name):
         if name is None:
             assert (parent is None)
             return None
-        return os.path.join(self.cset.entries[parent].path, name)
+        return pathjoin(self.cset.entries[parent].path, name)
 
     def add_file(self, id, parent, name, contents, mode):
         self.base.add_file(id, parent, name, contents, mode)
@@ -328,7 +328,7 @@
                 if parent is None:
                     return orig_inventory[file_id]
                 dirname = new_path(parent)
-                return os.path.join(dirname, orig_inventory[file_id])
+                return pathjoin(dirname, os.path.basename(orig_inventory[file_id]))
 
         new_inventory = {}
         for file_id in orig_inventory.iterkeys():
@@ -343,11 +343,11 @@
             new_inventory[file_id] = path
         return new_inventory
 
-    def apply_changeset(self, cset, conflict_handler=None, reverse=False):
+    def apply_changeset(self, cset, conflict_handler=None):
         inventory_change = changeset.apply_changeset(cset,
                                                      self.this.inventory_dict,
                                                      self.this.dir,
-                                                     conflict_handler, reverse)
+                                                     conflict_handler)
         self.this.inventory_dict =  self.apply_inv_change(inventory_change, 
                                                      self.this.inventory_dict)
 
@@ -355,7 +355,7 @@
         shutil.rmtree(self.dir)
 
 
-class MergeTest(unittest.TestCase):
+class MergeTest(TestCase):
     def test_change_name(self):
         """Test renames"""
         builder = MergeBuilder()
@@ -366,18 +366,18 @@
         builder.add_file("3", "0", "name5", "hello3", 0755)
         builder.change_name("3", this="name6")
         cset = builder.merge_changeset(ApplyMerge3)
-        self.assert_(cset.entries["2"].is_boring())
-        self.assert_(cset.entries["1"].name == "name1")
-        self.assert_(cset.entries["1"].new_name == "name2")
-        self.assert_(cset.entries["3"].is_boring())
+        self.failUnless(cset.entries["2"].is_boring())
+        self.assertEqual(cset.entries["1"].name, "name1")
+        self.assertEqual(cset.entries["1"].new_name, "name2")
+        self.failUnless(cset.entries["3"].is_boring())
         for tree in (builder.this, builder.other, builder.base):
-            self.assert_(tree.dir != builder.dir and 
-                   tree.dir.startswith(builder.dir))
+            self.assertNotEqual(tree.dir, builder.dir)
+            self.assertStartsWith(tree.dir, builder.dir)
             for path in tree.inventory_dict.itervalues():
                 fullpath = tree.abs_path(path)
-                self.assert_(fullpath.startswith(tree.dir))
-                self.assert_(not path.startswith(tree.dir))
-                self.assert_(os.path.exists(fullpath))
+                self.assertStartsWith(fullpath, tree.dir)
+                self.failIf(path.startswith(tree.dir))
+                self.failUnless(os.path.lexists(fullpath))
         builder.apply_changeset(cset)
         builder.cleanup()
         builder = MergeBuilder()
@@ -671,3 +671,168 @@
         merge(['b', -1], ['b', 0], this_dir='a')
         self.assert_(os.path.exists('a/file'))
 
+    def test_merge_swapping_renames(self):
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/un','wb').write('UN')
+        file('a/deux','wb').write('DEUX')
+        a_wt = a.working_tree()
+        a_wt.add('un')
+        a_wt.add('deux')
+        a_wt.commit('r0')
+        copy_branch(a,'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        b_wt.rename_one('un','tmp')
+        b_wt.rename_one('deux','un')
+        b_wt.rename_one('tmp','deux')
+        b_wt.commit('r1')
+        merge(['b', -1],['b', 1],this_dir='a')
+        self.assert_(os.path.exists('a/un'))
+        self.assert_(os.path.exists('a/deux'))
+        self.assertFalse(os.path.exists('a/tmp'))
+        self.assertEqual(file('a/un').read(),'DEUX')
+        self.assertEqual(file('a/deux').read(),'UN')
+
+    def test_merge_delete_and_add_same(self):
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/file', 'wb').write('THIS')
+        a_wt = a.working_tree()
+        a_wt.add('file')
+        a_wt.commit('r0')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        os.remove('b/file')
+        b_wt.commit('r1')
+        file('b/file', 'wb').write('THAT')
+        b_wt.add('file')
+        b_wt.commit('r2')
+        merge(['b', -1],['b', 1],this_dir='a')
+        self.assert_(os.path.exists('a/file'))
+        self.assertEqual(file('a/file').read(),'THAT')
+
+    def test_merge_rename_before_create(self):
+        """rename before create
+        
+        This case requires that you must not do creates
+        before move-into-place:
+
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        $ bzr mv foo bar
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/foo', 'wb').write('A/FOO')
+        a_wt = a.working_tree()
+        a_wt.add('foo')
+        a_wt.commit('added foo')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        b_wt.rename_one('foo', 'bar')
+        file('b/foo', 'wb').write('B/FOO')
+        b_wt.add('foo')
+        b_wt.commit('moved foo to bar, added new foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_create_before_rename(self):
+        """create before rename, target parents before children
+
+        This case requires that you must not do move-into-place
+        before creates, and that you must not do children after
+        parents:
+
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        $ bzr mkdir bar
+        $ bzr add bar
+        $ bzr mv foo bar/foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/foo', 'wb').write('A/FOO')
+        a_wt = a.working_tree()
+        a_wt.add('foo')
+        a_wt.commit('added foo')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        os.mkdir('b/bar')
+        b_wt.add('bar')
+        b_wt.rename_one('foo', 'bar/foo')
+        b_wt.commit('created bar dir, moved foo into bar')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_rename_to_temp_before_delete(self):
+        """rename to temp before delete, source children before parents
+
+        This case requires that you must not do deletes before
+        move-out-of-the-way, and that you must not do children
+        after parents:
+        
+        $ mkdir foo
+        $ touch foo/bar
+        $ bzr add foo/bar
+        $ bzr commit
+        $ bzr mv foo/bar bar
+        $ rmdir foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        os.mkdir('a/foo')
+        file('a/foo/bar', 'wb').write('A/FOO/BAR')
+        a_wt = a.working_tree()
+        a_wt.add('foo')
+        a_wt.add('foo/bar')
+        a_wt.commit('added foo/bar')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        b_wt.rename_one('foo/bar', 'bar')
+        os.rmdir('b/foo')
+        b_wt.remove('foo')
+        b_wt.commit('moved foo/bar to bar, deleted foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_delete_before_rename_to_temp(self):
+        """delete before rename to temp
+
+        This case requires that you must not do
+        move-out-of-the-way before deletes:
+        
+        $ touch foo
+        $ touch bar
+        $ bzr add foo bar
+        $ bzr commit
+        $ rm foo
+        $ bzr rm foo
+        $ bzr mv bar foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/foo', 'wb').write('A/FOO')
+        file('a/bar', 'wb').write('A/BAR')
+        a_wt = a.working_tree()
+        a_wt.add('foo')
+        a_wt.add('bar')
+        a_wt.commit('added foo and bar')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        os.unlink('b/foo')
+        b_wt.remove('foo')
+        b_wt.rename_one('bar', 'foo')
+        b_wt.commit('deleted foo, renamed bar to foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+

=== added file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_osutils.py	2006-01-02 22:37:32 +0000
@@ -0,0 +1,63 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the osutils wrapper.
+"""
+
+import os
+import sys
+
+import bzrlib
+import bzrlib.osutils as osutils
+from bzrlib.tests import TestCaseInTempDir
+
+
+class TestOSUtils(TestCaseInTempDir):
+
+    def test_fancy_rename(self):
+        # This should work everywhere
+        def rename(a, b):
+            osutils.fancy_rename(a, b,
+                    rename_func=os.rename,
+                    unlink_func=os.unlink)
+
+        open('a', 'wb').write('something in a\n')
+        rename('a', 'b')
+        self.failIfExists('a')
+        self.failUnlessExists('b')
+        self.check_file_contents('b', 'something in a\n')
+
+        open('a', 'wb').write('new something in a\n')
+        rename('b', 'a')
+
+        self.check_file_contents('a', 'something in a\n')
+
+    def test_rename(self):
+        # Rename should be semi-atomic on all platforms
+        open('a', 'wb').write('something in a\n')
+        osutils.rename('a', 'b')
+        self.failIfExists('a')
+        self.failUnlessExists('b')
+        self.check_file_contents('b', 'something in a\n')
+
+        open('a', 'wb').write('new something in a\n')
+        osutils.rename('b', 'a')
+
+        self.check_file_contents('a', 'something in a\n')
+
+
+    # TODO: test fancy_rename using a MemoryTransport
+

=== modified file 'bzrlib/tests/test_parent.py'
--- bzrlib/tests/test_parent.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_parent.py	2005-12-01 18:16:47 +0000
@@ -19,6 +19,7 @@
 from bzrlib.tests import TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.clone import copy_branch
+from bzrlib.osutils import abspath, realpath
 
 
 """Tests for Branch parent URL"""
@@ -52,7 +53,7 @@
         copy_branch(branch_from, 'to', None)
 
         branch_to = Branch.open('to')
-        abspath = os.path.abspath('from')
-        self.assertEquals(branch_to.get_parent(), abspath)
+        abs = abspath('from')
+        self.assertEquals(branch_to.get_parent(), abs)
         
 

=== added file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_permissions.py	2006-01-02 22:37:32 +0000
@@ -0,0 +1,323 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for bzr setting permissions.
+
+Files which are created underneath .bzr/ should inherit its permissions.
+So if the directory is group writable, the files and subdirs should be as well.
+
+In the future, when we have Repository/Branch/Checkout information, the
+permissions should be inherited individually, rather than all be the same.
+
+TODO: jam 20051215 There are no tests for ftp yet, because we have no ftp server
+TODO: jam 20051215 Currently the default behavior for 'bzr branch' is just 
+                   defined by the local umask. This isn't terrible, is it
+                   the truly desired behavior?
+"""
+
+import os
+import sys
+import stat
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.tests.test_transport import check_mode
+
+
+def chmod_r(base, file_mode, dir_mode):
+    """Recursively chmod from a base directory"""
+    assert os.path.isdir(base)
+    os.chmod(base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            os.chmod(p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            os.chmod(p, file_mode)
+
+
+def check_mode_r(test, base, file_mode, dir_mode, include_base=True):
+    """Check that all permissions match
+
+    :param test: The TestCase being run
+    :param base: The path to the root directory to check
+    :param file_mode: The mode for all files
+    :param dir_mode: The mode for all directories
+    :param include_base: If false, only check the subdirectories
+    """
+    assert os.path.isdir(base)
+    if include_base:
+        check_mode(test, base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            check_mode(test, p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            check_mode(test, p, file_mode)
+
+
+def assertEqualMode(test, mode, mode_test):
+    test.assertEqual(mode, mode_test,
+                     'mode mismatch %o != %o' % (mode, mode_test))
+
+
+class TestPermissions(TestCaseInTempDir):
+
+    def test_new_files(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        open('a', 'wb').write('foo\n')
+        t.add('a')
+        t.commit('foo')
+
+        # Delete them because we are modifying the filesystem underneath them
+        del b, t 
+        chmod_r('.bzr', 0644, 0755)
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 0755, b._dir_mode)
+        assertEqualMode(self, 0644, b._file_mode)
+
+        # Modifying a file shouldn't break the permissions
+        open('a', 'wb').write('foo2\n')
+        t.commit('foo2')
+        # The mode should be maintained after commit
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        # Adding a new file should maintain the permissions
+        open('b', 'wb').write('new b\n')
+        t.add('b')
+        t.commit('new b')
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        del b, t
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 0775)
+        check_mode_r(self, '.bzr', 0664, 0775)
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 0775, b._dir_mode)
+        assertEqualMode(self, 0664, b._file_mode)
+
+        open('a', 'wb').write('foo3\n')
+        t.commit('foo3')
+        check_mode_r(self, '.bzr', 0664, 0775)
+
+        open('c', 'wb').write('new c\n')
+        t.add('c')
+        t.commit('new c')
+        check_mode_r(self, '.bzr', 0664, 0775)
+
+        # Test the group sticky bit
+        del b, t
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 02775)
+        check_mode_r(self, '.bzr', 0664, 02775)
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 02775, b._dir_mode)
+        assertEqualMode(self, 0664, b._file_mode)
+
+        open('a', 'wb').write('foo4\n')
+        t.commit('foo4')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+        open('d', 'wb').write('new d\n')
+        t.add('d')
+        t.commit('new d')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+    def test_disable_set_mode(self):
+        # TODO: jam 20051215 Ultimately, this test should probably test that
+        #                    extra chmod calls aren't being made
+        import bzrlib.branch
+        try:
+            b = Branch.initialize(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_dir_mode = False
+            b = Branch.open(u'.')
+            self.assertEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_file_mode = False
+            b = Branch.open(u'.')
+            self.assertEqual(None, b._dir_mode)
+            self.assertEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_dir_mode = True
+            b = Branch.open(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_file_mode = True
+            b = Branch.open(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+        finally:
+            bzrlib.branch.BzrBranch._set_dir_mode = True
+            bzrlib.branch.BzrBranch._set_file_mode = True
+
+    def test_new_branch(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        os.mkdir('a')
+        mode = stat.S_IMODE(os.stat('a').st_mode)
+        b = Branch.initialize('a')
+        assertEqualMode(self, mode, b._dir_mode)
+        assertEqualMode(self, mode & ~07111, b._file_mode)
+
+        os.mkdir('b')
+        os.chmod('b', 02777)
+        b = Branch.initialize('b')
+        assertEqualMode(self, 02777, b._dir_mode)
+        assertEqualMode(self, 00666, b._file_mode)
+        check_mode_r(self, 'b/.bzr', 00666, 02777)
+
+        os.mkdir('c')
+        os.chmod('c', 02750)
+        b = Branch.initialize('c')
+        assertEqualMode(self, 02750, b._dir_mode)
+        assertEqualMode(self, 00640, b._file_mode)
+        check_mode_r(self, 'c/.bzr', 00640, 02750)
+
+        os.mkdir('d')
+        os.chmod('d', 0700)
+        b = Branch.initialize('d')
+        assertEqualMode(self, 0700, b._dir_mode)
+        assertEqualMode(self, 0600, b._file_mode)
+        check_mode_r(self, 'd/.bzr', 00600, 0700)
+
+
+class TestSftpPermissions(TestCaseWithSFTPServer):
+
+    def test_new_files(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        # Though it would be nice to test that SFTP to a server
+        # which does support chmod has the right effect
+
+        from bzrlib.transport.sftp import SFTPTransport
+
+        # We don't actually use it directly, we just want to
+        # keep the connection open, since StubSFTPServer only
+        # allows 1 connection
+        self.delayed_setup()
+        _transport = SFTPTransport(self._sftp_url)
+
+        os.mkdir('local')
+        b_local = Branch.initialize(u'local')
+        t_local = b_local.working_tree()
+        open('local/a', 'wb').write('foo\n')
+        t_local.add('a')
+        t_local.commit('foo')
+
+        # Delete them because we are modifying the filesystem underneath them
+        del b_local, t_local 
+        chmod_r('local/.bzr', 0644, 0755)
+        check_mode_r(self, 'local/.bzr', 0644, 0755)
+
+        b_local = Branch.open(u'local')
+        t_local = b_local.working_tree()
+        assertEqualMode(self, 0755, b_local._dir_mode)
+        assertEqualMode(self, 0644, b_local._file_mode)
+
+        os.mkdir('sftp')
+        # Why does self._sftp_url end with a slash????
+        sftp_url = self._sftp_url + 'sftp'
+        b_sftp = Branch.initialize(sftp_url)
+
+        b_sftp.pull(b_local)
+        del b_sftp
+        chmod_r('sftp/.bzr', 0644, 0755)
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        b_sftp = Branch.open(sftp_url)
+        assertEqualMode(self, 0755, b_sftp._dir_mode)
+        assertEqualMode(self, 0644, b_sftp._file_mode)
+
+        open('local/a', 'wb').write('foo2\n')
+        t_local.commit('foo2')
+        b_sftp.pull(b_local)
+        # The mode should be maintained after commit
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        open('local/b', 'wb').write('new b\n')
+        t_local.add('b')
+        t_local.commit('new b')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        del b_sftp
+        # Recursively update the modes of all files
+        chmod_r('sftp/.bzr', 0664, 0775)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+        b_sftp = Branch.open(sftp_url)
+        assertEqualMode(self, 0775, b_sftp._dir_mode)
+        assertEqualMode(self, 0664, b_sftp._file_mode)
+
+        open('local/a', 'wb').write('foo3\n')
+        t_local.commit('foo3')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+        open('local/c', 'wb').write('new c\n')
+        t_local.add('c')
+        t_local.commit('new c')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+    def test_sftp_server_modes(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        umask = 0022
+        original_umask = os.umask(umask)
+
+        try:
+            from bzrlib.transport.sftp import SFTPTransport
+            self.delayed_setup()
+            t = SFTPTransport(self._sftp_url)
+            # Direct access should be masked by umask
+            t._sftp_open_exclusive('a', mode=0666).write('foo\n')
+            check_mode(self, 'a', 0666 &~umask)
+
+            # but Transport overrides umask
+            t.put('b', 'txt', mode=0666)
+            check_mode(self, 'b', 0666)
+
+            t._sftp.mkdir('c', mode=0777)
+            check_mode(self, 'c', 0777 &~umask)
+
+            t.mkdir('d', mode=0777)
+            check_mode(self, 'd', 0777)
+        finally:
+            os.umask(original_umask)
+
+

=== modified file 'bzrlib/tests/test_plugins.py'
--- bzrlib/tests/test_plugins.py	2005-11-29 23:20:31 +0000
+++ bzrlib/tests/test_plugins.py	2005-12-16 03:43:14 +0000
@@ -26,13 +26,14 @@
 import bzrlib.plugin
 import bzrlib.plugins
 from bzrlib.tests import TestCaseInTempDir
+from bzrlib.osutils import pathjoin, abspath
 
 class PluginTest(TestCaseInTempDir):
     """Create an external plugin and test loading."""
 #    def test_plugin_loading(self):
 #        orig_help = self.run_bzr_captured('bzr help commands')[0]
 #        os.mkdir('plugin_test')
-#        f = open(os.path.join('plugin_test', 'myplug.py'), 'wt')
+#        f = open(pathjoin('plugin_test', 'myplug.py'), 'wt')
 #        f.write(PLUGIN_TEXT)
 #        f.close()
 #        newhelp = self.run_bzr_captured('bzr help commands')[0]
@@ -45,7 +46,7 @@
 #        shutil.rmtree('plugin_test')
 #
 
-#         os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
+#         os.environ['BZRPLUGINPATH'] = abspath('plugin_test')
 #         help = backtick('bzr help commands')
 #         assert help.find('myplug') != -1
 #         assert help.find('Just a simple test plugin.') != -1
@@ -54,7 +55,7 @@
 #         assert backtick('bzr myplug') == 'Hello from my plugin\n'
 #         assert backtick('bzr mplg') == 'Hello from my plugin\n'
 
-#         f = open(os.path.join('plugin_test', 'override.py'), 'wb')
+#         f = open(pathjoin('plugin_test', 'override.py'), 'wb')
 #         f.write("""import bzrlib, bzrlib.commands
 #     class cmd_commit(bzrlib.commands.cmd_commit):
 #         '''Commit changes into a new revision.'''

=== modified file 'bzrlib/tests/test_revisionnamespaces.py'
--- bzrlib/tests/test_revisionnamespaces.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_revisionnamespaces.py	2005-12-30 20:54:47 +0000
@@ -80,6 +80,10 @@
         self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
                           'b@r-0-4')
 
+        # This should be in the revision store, but not in revision-history
+        self.assertEquals((None, 'b@r-0-4'),
+                RevisionSpec('revid:b@r-0-4').in_history(b))
+
     def test_branch_namespace(self):
         """Ensure that the branch namespace pulls in the requisite content."""
         self.build_tree(['branch1/', 'branch1/file', 'branch2/'])

=== modified file 'bzrlib/tests/test_setup.py'
--- bzrlib/tests/test_setup.py	2005-12-08 12:01:09 +0000
+++ bzrlib/tests/test_setup.py	2005-12-16 04:53:00 +0000
@@ -1,6 +1,8 @@
 """ test for setup.py build process """
 
 import os
+import sys
+import subprocess
 import shutil
 
 from bzrlib.tests import TestCase
@@ -14,10 +16,10 @@
     def test_build(self):
         """ test cmd `python setup.py build` """
         # run setup.py build as subproces and catch return code
-        p = os.popen("python setup.py build")
-        s = p.readlines()
-        res = p.close()
-        self.assertEqual(res, None, '`python setup.py build` fails')
+        p = subprocess.Popen([sys.executable, 'setup.py', 'build'],
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        s = p.communicate()
+        self.assertEqual(0, p.returncode, '`python setup.py build` fails')
 
     def tearDown(self):
         """ cleanup build directory """

=== renamed file 'bzrlib/tests/test_sftp.py' => 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp.py	2005-12-01 21:31:42 +0000
+++ bzrlib/tests/test_sftp_transport.py	2006-01-02 22:37:32 +0000
@@ -18,9 +18,10 @@
 import socket
 import threading
 
-from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
 from bzrlib.tests.test_transport import TestTransportMixIn
 import bzrlib.errors as errors
+from bzrlib.osutils import pathjoin, lexists
 
 try:
     import paramiko
@@ -90,7 +91,7 @@
     
     def _run_server(self, s, stop_event):
         ssh_server = paramiko.Transport(s)
-        key_file = os.path.join(self._root, 'test_rsa.key')
+        key_file = pathjoin(self._root, 'test_rsa.key')
         file(key_file, 'w').write(STUB_SERVER_KEY)
         host_key = paramiko.RSAKey.from_private_key_file(key_file)
         ssh_server.add_server_key(host_key)
@@ -102,6 +103,8 @@
         stop_event.wait(30.0)
 
     def setUp(self):
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
         TestCaseInTempDir.setUp(self)
         self._root = self.test_dir
         self._is_setup = False
@@ -157,7 +160,7 @@
         self.assertRaises(LockError, t.lock_write, 'bogus')
 
         l.unlock()
-        self.failIf(os.path.lexists('bogus.write-lock'))
+        self.failIf(lexists('bogus.write-lock'))
 
         open('something.write-lock', 'wb').write('fake lock\n')
         self.assertRaises(LockError, t.lock_write, 'something')
@@ -189,6 +192,11 @@
 
 
 class SFTPNonServerTest(TestCase):
+    def setUp(self):
+        TestCase.setUp(self)
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
+
     def test_parse_url(self):
         from bzrlib.transport.sftp import SFTPTransport
         s = SFTPTransport('sftp://simple.example.com/%2fhome/source', clone_from=fake)
@@ -270,11 +278,11 @@
         self.failUnlessExists('.bzr/branch-format')
         self.failUnlessExists('.bzr/branch-lock')
 
-        self.failIf(os.path.lexists('.bzr/branch-lock.write-lock'))
+        self.failIf(lexists('.bzr/branch-lock.write-lock'))
         b.lock_write()
         self.failUnlessExists('.bzr/branch-lock.write-lock')
         b.unlock()
-        self.failIf(os.path.lexists('.bzr/branch-lock.write-lock'))
+        self.failIf(lexists('.bzr/branch-lock.write-lock'))
 
     def test_no_working_tree(self):
         from bzrlib.branch import Branch
@@ -298,9 +306,10 @@
 
         self.assertEquals(b2.revision_history(), ['a1'])
 
-
-if not paramiko_loaded:
-    # TODO: Skip these
-    del SFTPTransportTest
-    del SFTPNonServerTest
-    del SFTPBranchTest
+        open('a/foo', 'wt').write('something new in foo\n')
+        t.commit('new', rev_id='a2')
+        b2.pull(b)
+
+        self.assertEquals(b2.revision_history(), ['a1', 'a2'])
+
+

=== modified file 'bzrlib/tests/test_store.py'
--- bzrlib/tests/test_store.py	2005-12-01 21:31:42 +0000
+++ bzrlib/tests/test_store.py	2005-12-18 04:08:46 +0000
@@ -92,7 +92,8 @@
         self.assertEqual(store.total_size(), (2, 55))
         
     def test__relpath_suffixed(self):
-        my_store = TextStore(MockTransport(), True, compressed=True)
+        my_store = TextStore(MockTransport(),
+                             prefixed=True, compressed=True)
         my_store.register_suffix('dsc')
         self.assertEqual('45/foo.dsc', my_store._relpath('foo', ['dsc']))
 
@@ -357,7 +358,8 @@
                          my_store.get('missing', 'sig').read())
 
     def test___iter__no_suffix(self):
-        my_store = TextStore(MemoryTransport(), False, compressed=False)
+        my_store = TextStore(MemoryTransport(),
+                             prefixed=False, compressed=False)
         stream = StringIO("content")
         my_store.add(stream, "foo")
         self.assertEqual(set(['foo']),
@@ -382,7 +384,8 @@
 
     def test_copy_suffixes(self):
         from_store = self.get_populated_store()
-        to_store = TextStore(MemoryTransport(), True, compressed=True)
+        to_store = TextStore(MemoryTransport(),
+                             prefixed=True, compressed=True)
         to_store.register_suffix('sig')
         copy_all(from_store, to_store)
         self.assertEqual(1, len(to_store))

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2005-12-05 12:04:37 +0000
+++ bzrlib/tests/test_transport.py	2005-12-18 04:08:46 +0000
@@ -16,6 +16,8 @@
 
 
 import os
+import sys
+import stat
 from cStringIO import StringIO
 
 from bzrlib.errors import (NoSuchFile, FileExists,
@@ -23,6 +25,7 @@
 from bzrlib.tests import TestCase, TestCaseInTempDir
 from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
 from bzrlib.transport import memory, urlescape
+from bzrlib.osutils import pathjoin
 
 
 def _append(fn, txt):
@@ -33,6 +36,21 @@
     f.close()
     del f
 
+
+if sys.platform != 'win32':
+    def check_mode(test, path, mode):
+        """Check that a particular path has the correct mode."""
+        actual_mode = stat.S_IMODE(os.stat(path).st_mode)
+        test.assertEqual(mode, actual_mode,
+            'mode of %r incorrect (%o != %o)' % (path, mode, actual_mode))
+else:
+    def check_mode(test, path, mode):
+        """On win32 chmod doesn't have any effect, 
+        so don't actually check anything
+        """
+        return
+
+
 class TestTransport(TestCase):
     """Test the non transport-concrete class functionality."""
 
@@ -95,11 +113,11 @@
         self.assertEqual(open('a', 'rb').read(), t.get('a').read())
         content_f = t.get_multi(files)
         for path,f in zip(files, content_f):
-            self.assertEqual(open(path).read(), f.read())
+            self.assertEqual(f.read(), open(path, 'rb').read())
 
         content_f = t.get_multi(iter(files))
         for path,f in zip(files, content_f):
-            self.assertEqual(f.read(), open(path).read())
+            self.assertEqual(f.read(), open(path, 'rb').read())
 
         self.assertRaises(NoSuchFile, t.get, 'c')
         self.assertListRaises(NoSuchFile, t.get_multi, ['a', 'b', 'c'])
@@ -108,6 +126,14 @@
     def test_put(self):
         t = self.get_transport()
 
+        # TODO: jam 20051215 No need to do anything if the test is readonly
+        #                    origininally it was thought that it would give
+        #                    more of a workout to readonly tests. By now the
+        #                    suite is probably thorough enough without testing
+        #                    readonly protocols in write sections
+        #                    The only thing that needs to be tested is that the
+        #                    right error is raised
+
         if self.readonly:
             self.assertRaises(TransportNotPossible,
                     t.put, 'a', 'some text for a\n')
@@ -158,6 +184,26 @@
             self.assertRaises(NoSuchFile,
                     t.put, 'path/doesnt/exist/c', 'contents')
 
+        if not self.readonly:
+            t.put('mode644', 'test text\n', mode=0644)
+            check_mode(self, 'mode644', 0644)
+
+            t.put('mode666', 'test text\n', mode=0666)
+            check_mode(self, 'mode666', 0666)
+
+            t.put('mode600', 'test text\n', mode=0600)
+            check_mode(self, 'mode600', 0600)
+
+            # Yes, you can put a file such that it becomes readonly
+            t.put('mode400', 'test text\n', mode=0400)
+            check_mode(self, 'mode400', 0400)
+
+            t.put_multi([('mmode644', 'text\n')], mode=0644)
+            check_mode(self, 'mmode644', 0644)
+
+        # TODO: jam 20051215 test put_multi with a mode. I didn't bother because
+        #                    it seems most people don't like the _multi functions
+
     def test_put_file(self):
         t = self.get_transport()
 
@@ -212,6 +258,24 @@
         self.check_file_contents('f5', 'here is some text\nand a bit more\n')
         self.check_file_contents('f6', 'some text for the\nthird file created\n')
 
+        if not self.readonly:
+            sio = StringIO('test text\n')
+            t.put('mode644', sio, mode=0644)
+            check_mode(self, 'mode644', 0644)
+
+            a = open('mode644', 'rb')
+            t.put('mode666', a, mode=0666)
+            check_mode(self, 'mode666', 0666)
+
+            a = open('mode644', 'rb')
+            t.put('mode600', a, mode=0600)
+            check_mode(self, 'mode600', 0600)
+
+            # Yes, you can put a file such that it becomes readonly
+            a = open('mode644', 'rb')
+            t.put('mode400', a, mode=0400)
+            check_mode(self, 'mode400', 0400)
+
     def test_mkdir(self):
         t = self.get_transport()
 
@@ -278,7 +342,26 @@
                              ('dir_b/b', 'contents of dir_b/b')])
                           , 2)
         for f in ('dir_a/a', 'dir_b/b'):
-            self.assertEqual(t.get(f).read(), open(f).read())
+            self.assertEqual(t.get(f).read(), open(f, 'rb').read())
+
+        if not self.readonly:
+            # Test mkdir with a mode
+            t.mkdir('dmode755', mode=0755)
+            check_mode(self, 'dmode755', 0755)
+
+            t.mkdir('dmode555', mode=0555)
+            check_mode(self, 'dmode555', 0555)
+
+            t.mkdir('dmode777', mode=0777)
+            check_mode(self, 'dmode777', 0777)
+
+            t.mkdir('dmode700', mode=0700)
+            check_mode(self, 'dmode700', 0700)
+
+            # TODO: jam 20051215 test mkdir_multi with a mode
+            t.mkdir_multi(['mdmode755'], mode=0755)
+            check_mode(self, 'mdmode755', 0755)
+
 
     def test_copy_to(self):
         import tempfile
@@ -289,14 +372,16 @@
         files = ['a', 'b', 'c', 'd']
         self.build_tree(files)
 
-        dtmp = tempfile.mkdtemp(dir=u'.', prefix='test-transport-')
-        dtmp_base = os.path.basename(dtmp)
-        local_t = LocalTransport(dtmp)
+        def get_temp_local():
+            dtmp = tempfile.mkdtemp(dir=u'.', prefix='test-transport-')
+            dtmp_base = os.path.basename(dtmp)
+            return dtmp_base, LocalTransport(dtmp)
+        dtmp_base, local_t = get_temp_local()
 
         t.copy_to(files, local_t)
         for f in files:
-            self.assertEquals(open(f).read(),
-                    open(os.path.join(dtmp_base, f)).read())
+            self.assertEquals(open(f, 'rb').read(),
+                    open(pathjoin(dtmp_base, f), 'rb').read())
 
         # Test that copying into a missing directory raises
         # NoSuchFile
@@ -304,22 +389,26 @@
         open('e/f', 'wb').write('contents of e')
         self.assertRaises(NoSuchFile, t.copy_to, ['e/f'], local_t)
 
-        os.mkdir(os.path.join(dtmp_base, 'e'))
+        os.mkdir(pathjoin(dtmp_base, 'e'))
         t.copy_to(['e/f'], local_t)
 
-        del dtmp, dtmp_base, local_t
+        del dtmp_base, local_t
 
-        dtmp = tempfile.mkdtemp(dir=u'.', prefix='test-transport-')
-        dtmp_base = os.path.basename(dtmp)
-        local_t = LocalTransport(dtmp)
+        dtmp_base, local_t = get_temp_local()
 
         files = ['a', 'b', 'c', 'd']
         t.copy_to(iter(files), local_t)
         for f in files:
-            self.assertEquals(open(f).read(),
-                    open(os.path.join(dtmp_base, f)).read())
-
-        del dtmp, dtmp_base, local_t
+            self.assertEquals(open(f, 'rb').read(),
+                    open(pathjoin(dtmp_base, f), 'rb').read())
+
+        del dtmp_base, local_t
+
+        for mode in (0666, 0644, 0600, 0400):
+            dtmp_base, local_t = get_temp_local()
+            t.copy_to(files, local_t, mode=mode)
+            for f in files:
+                check_mode(self, os.path.join(dtmp_base, f), mode)
 
     def test_append(self):
         t = self.get_transport()

=== modified file 'bzrlib/tests/test_weave.py'
--- bzrlib/tests/test_weave.py	2005-11-23 15:44:24 +0000
+++ bzrlib/tests/test_weave.py	2005-12-20 02:45:27 +0000
@@ -38,7 +38,6 @@
           "A second line"]
 
 
-
 class TestBase(TestCase):
     def check_read_write(self, k):
         """Check the weave k can be written & re-read."""
@@ -85,7 +84,6 @@
         self.assertEqual(idx, 0)
 
 
-
 class AnnotateOne(TestBase):
     def runTest(self):
         k = Weave()
@@ -108,7 +106,6 @@
         self.assertEqual(k.get(1), TEXT_1)
 
 
-
 class AddWithGivenSha(TestBase):
     def runTest(self):
         """Add with caller-supplied SHA-1"""
@@ -118,6 +115,17 @@
         k.add('text0', [], [t], sha1=sha_string(t))
 
 
+class GetSha1(TestBase):
+    def test_get_sha1(self):
+        k = Weave()
+        k.add('text0', [], 'text0')
+        self.assertEqual('34dc0e430c642a26c3dd1c2beb7a8b4f4445eb79',
+                         k.get_sha1('text0'))
+        self.assertRaises(errors.WeaveRevisionNotPresent,
+                          k.get_sha1, 0)
+        self.assertRaises(errors.WeaveRevisionNotPresent,
+                          k.get_sha1, 'text1')
+                        
 
 class InvalidAdd(TestBase):
     """Try to use invalid version number during add."""
@@ -140,7 +148,6 @@
         self.assertEqual(idx, idx2)
 
 
-
 class InvalidRepeatedAdd(TestBase):
     def runTest(self):
         k = Weave()
@@ -157,7 +164,6 @@
                           TEXT_0)
         
 
-
 class InsertLines(TestBase):
     """Store a revision that adds one line to the original.
 
@@ -214,7 +220,6 @@
                           (4, 'ccc')])
 
 
-
 class DeleteLines(TestBase):
     """Deletion of lines from existing text.
 
@@ -244,8 +249,6 @@
         for i in range(len(texts)):
             self.assertEqual(k.get(i+1),
                              texts[i])
-            
-
 
 
 class SuicideDelete(TestBase):
@@ -271,7 +274,6 @@
                           0)        
 
 
-
 class CannedDelete(TestBase):
     """Unpack canned weave with deleted lines."""
     def runTest(self):
@@ -288,6 +290,8 @@
                 'last line',
                 ('}', 0),
                 ]
+        k._sha1s = [sha_string('first lineline to be deletedlast line')
+                  , sha_string('first linelast line')]
 
         self.assertEqual(k.get(0),
                          ['first line',
@@ -301,7 +305,6 @@
                           ])
 
 
-
 class CannedReplacement(TestBase):
     """Unpack canned weave with deleted lines."""
     def runTest(self):
@@ -321,6 +324,8 @@
                 'last line',
                 ('}', 0),
                 ]
+        k._sha1s = [sha_string('first lineline to be deletedlast line')
+                  , sha_string('first linereplacement linelast line')]
 
         self.assertEqual(k.get(0),
                          ['first line',
@@ -335,7 +340,6 @@
                           ])
 
 
-
 class BadWeave(TestBase):
     """Test that we trap an insert which should not occur."""
     def runTest(self):
@@ -421,6 +425,12 @@
                 '}',
                 ('}', 0)]
 
+        k._sha1s = [sha_string('foo {}')
+                  , sha_string('foo {  added in version 1  also from v1}')
+                  , sha_string('foo {  added in v2}')
+                  , sha_string('foo {  added in version 1  added in v2  also from v1}')
+                  ]
+
         self.assertEqual(k.get(0),
                          ['foo {',
                           '}'])
@@ -444,7 +454,6 @@
                           '}'])
                          
 
-
 class DeleteLines2(TestBase):
     """Test recording revisions that delete lines.
 
@@ -472,7 +481,6 @@
                           (0, "fine")])
 
 
-
 class IncludeVersions(TestBase):
     """Check texts that are stored across multiple revisions.
 
@@ -494,6 +502,9 @@
                 "second line",
                 ('}', 1)]
 
+        k._sha1s = [sha_string('first line')
+                  , sha_string('first linesecond line')]
+
         self.assertEqual(k.get(1),
                          ["first line",
                           "second line"])
@@ -523,6 +534,10 @@
                 ('}', 2),                
                 ]
 
+        k._sha1s = [sha_string('first line')
+                  , sha_string('first linesecond line')
+                  , sha_string('first linealternative second line')]
+
         self.assertEqual(k.get(0),
                          ["first line"])
 
@@ -538,7 +553,6 @@
                          [0, 2])
 
 
-
 class ReplaceLine(TestBase):
     def runTest(self):
         k = Weave()
@@ -555,7 +569,6 @@
         self.assertEqual(k.get(1), text1)
 
 
-
 class Merge(TestBase):
     """Storage of versions that merge diverged parents"""
     def runTest(self):
@@ -612,7 +625,6 @@
                            [['bbb']]])
 
 
-
 class NonConflict(TestBase):
     """Two descendants insert compatible changes.
 
@@ -625,9 +637,6 @@
         k.add([0], ['111', 'aaa', 'ccc', 'bbb'])
         k.add([1], ['aaa', 'ccc', 'bbb', '222'])
 
-    
-    
-
 
 class AutoMerge(TestBase):
     def runTest(self):
@@ -651,7 +660,6 @@
                           'line from 1',
                           'bbb',
                           'line from 2', 'more from 2'])
-        
 
 
 class Khayyam(TestBase):
@@ -701,7 +709,6 @@
         self.check_read_write(k)
 
 
-
 class MergeCases(TestBase):
     def doMerge(self, base, a, b, mp):
         from cStringIO import StringIO
@@ -881,3 +888,106 @@
         eq = self.assertEquals
         eq(sorted(wa.iter_names()), ['v1', 'v2', 'v3', 'x1',])
         eq(wa.get_text('x1'), 'line from x1\n')
+
+
+class Corruption(TestCase):
+
+    def test_detection(self):
+        # Test weaves detect corruption.
+        #
+        # Weaves contain a checksum of their texts.
+        # When a text is extracted, this checksum should be
+        # verified.
+
+        w = Weave()
+        w.add('v1', [], ['hello\n'])
+        w.add('v2', ['v1'], ['hello\n', 'there\n'])
+
+        # We are going to invasively corrupt the text
+        # Make sure the internals of weave are the same
+        self.assertEqual([('{', 0)
+                        , 'hello\n'
+                        , ('}', None)
+                        , ('{', 1)
+                        , 'there\n'
+                        , ('}', None)
+                        ], w._weave)
+
+        self.assertEqual(['f572d396fae9206628714fb2ce00f72e94f2258f'
+                        , '90f265c6e75f1c8f9ab76dcf85528352c5f215ef'
+                        ], w._sha1s)
+        w.check()
+
+        # Corrupted
+        w._weave[4] = 'There\n'
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, list, w.get_iter('v2'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+        # Corrected
+        w._weave[4] = 'there\n'
+        self.assertEqual('hello\nthere\n', w.get_text('v2'))
+
+        #Invalid checksum, first digit changed
+        w._sha1s[1] =  'f0f265c6e75f1c8f9ab76dcf85528352c5f215ef'
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, list, w.get_iter('v2'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+    def test_written_detection(self):
+        # Test detection of weave file corruption.
+        #
+        # Make sure that we can detect if a weave file has
+        # been corrupted. This doesn't test all forms of corruption,
+        # but it at least helps verify the data you get, is what you want.
+        from cStringIO import StringIO
+
+        w = Weave()
+        w.add('v1', [], ['hello\n'])
+        w.add('v2', ['v1'], ['hello\n', 'there\n'])
+
+        tmpf = StringIO()
+        write_weave(w, tmpf)
+
+        # Because we are corrupting, we need to make sure we have the exact text
+        self.assertEquals('# bzr weave file v5\n'
+                          'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                          'i 0\n1 90f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                          'w\n{ 0\n. hello\n}\n{ 1\n. there\n}\nW\n',
+                          tmpf.getvalue())
+
+        # Change a single letter
+        tmpf = StringIO('# bzr weave file v5\n'
+                        'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                        'i 0\n1 90f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                        'w\n{ 0\n. hello\n}\n{ 1\n. There\n}\nW\n')
+
+        w = read_weave(tmpf)
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, list, w.get_iter('v2'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+        # Change the sha checksum
+        tmpf = StringIO('# bzr weave file v5\n'
+                        'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                        'i 0\n1 f0f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                        'w\n{ 0\n. hello\n}\n{ 1\n. there\n}\nW\n')
+
+        w = read_weave(tmpf)
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, list, w.get_iter('v2'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+

=== modified file 'bzrlib/tests/test_whitebox.py'
--- bzrlib/tests/test_whitebox.py	2005-12-01 21:31:42 +0000
+++ bzrlib/tests/test_whitebox.py	2005-12-01 21:49:24 +0000
@@ -4,6 +4,7 @@
 from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.branch import ScratchBranch, Branch
 from bzrlib.errors import PathNotChild
+from bzrlib.osutils import relpath, pathjoin, abspath, realpath
 
 
 class TestBranch(TestCaseInTempDir):
@@ -76,20 +77,19 @@
         job: given a path (either relative to cwd or absolute), work out
         if it is inside a branch and return the path relative to the base.
         """
-        from bzrlib.osutils import relpath
         import tempfile, shutil
         
         savedir = os.getcwdu()
         dtmp = tempfile.mkdtemp()
         # On Mac OSX, /tmp actually expands to /private/tmp
-        dtmp = os.path.realpath(dtmp)
+        dtmp = realpath(dtmp)
 
         def rp(p):
             return relpath(dtmp, p)
         
         try:
             # check paths inside dtmp while standing outside it
-            self.assertEqual(rp(os.path.join(dtmp, 'foo')), 'foo')
+            self.assertEqual(rp(pathjoin(dtmp, 'foo')), 'foo')
 
             # root = nothing
             self.assertEqual(rp(dtmp), '')
@@ -112,14 +112,13 @@
             # directory, or nearby
             os.chdir(dtmp)
 
-            FOO_BAR_QUUX = os.path.join('foo', 'bar', 'quux')
-            self.assertEqual(rp('foo/bar/quux'), FOO_BAR_QUUX)
+            self.assertEqual(rp('foo/bar/quux'), 'foo/bar/quux')
 
             self.assertEqual(rp('foo'), 'foo')
 
             self.assertEqual(rp('./foo'), 'foo')
 
-            self.assertEqual(rp(os.path.abspath('foo')), 'foo')
+            self.assertEqual(rp(abspath('foo')), 'foo')
 
             self.assertRaises(PathNotChild,
                               rp, '../foo')

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2005-11-30 00:25:50 +0000
+++ bzrlib/tests/test_workingtree.py	2006-01-02 22:54:45 +0000
@@ -20,6 +20,7 @@
 from bzrlib.errors import NotBranchError, NotVersionedError
 from bzrlib.tests import TestCaseInTempDir
 from bzrlib.trace import mutter
+from bzrlib.osutils import pathjoin, getcwd, has_symlinks
 from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
                                 WorkingTree)
 
@@ -53,12 +54,14 @@
         branch = Branch.initialize(u'.')
         os.mkdir('dir')
         print >> open('file', 'w'), "content"
-        os.symlink('target', 'symlink')
+        if has_symlinks():
+            os.symlink('target', 'symlink')
         tree = branch.working_tree()
         files = list(tree.list_files())
         self.assertEqual(files[0], ('dir', '?', 'directory', None, TreeDirectory()))
         self.assertEqual(files[1], ('file', '?', 'file', None, TreeFile()))
-        self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
+        if has_symlinks():
+            self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
 
     def test_open_containing(self):
         branch = Branch.initialize(u'.')
@@ -74,7 +77,7 @@
         # paths that are urls are just plain wrong for working trees.
         self.assertRaises(NotBranchError,
                           WorkingTree.open_containing, 
-                          'file:///' + os.getcwdu())
+                          'file:///' + getcwd())
 
     def test_construct_with_branch(self):
         branch = Branch.initialize(u'.')
@@ -93,7 +96,7 @@
         branch = Branch.initialize(u'.')
         tree = WorkingTree(branch.base)
         self.assertEqual('child',
-                         tree.relpath(os.path.join(os.getcwd(), 'child')))
+                         tree.relpath(pathjoin(getcwd(), 'child')))
 
     def test_lock_locks_branch(self):
         branch = Branch.initialize(u'.')
@@ -168,3 +171,17 @@
         self.assertEquals(list(tree.unknowns()),
                           ['hello.txt'])
 
+    def test_hashcache(self):
+        from bzrlib.tests.test_hashcache import pause
+        b = Branch.initialize(u'.')
+        tree = WorkingTree(u'.', b)
+        self.build_tree(['hello.txt',
+                         'hello.txt~'])
+        tree.add('hello.txt')
+        pause()
+        sha = tree.get_file_sha1(tree.path2id('hello.txt'))
+        self.assertEqual(1, tree._hashcache.miss_count)
+        tree2 = WorkingTree(u'.', b)
+        sha2 = tree2.get_file_sha1(tree2.path2id('hello.txt'))
+        self.assertEqual(0, tree2._hashcache.miss_count)
+        self.assertEqual(1, tree2._hashcache.hit_count)

=== modified file 'bzrlib/tests/treeshape.py'
--- bzrlib/tests/treeshape.py	2005-10-31 20:40:26 +0000
+++ bzrlib/tests/treeshape.py	2005-12-01 17:25:16 +0000
@@ -31,6 +31,7 @@
 import stat
 
 from bzrlib.trace import warning
+from bzrlib.osutils import pathjoin
 
 def build_tree_contents(template):
     """Reconstitute some files from a text description.
@@ -63,7 +64,7 @@
         yield (dirpath + '/', )
         filenames.sort()
         for fn in filenames:
-            fullpath = os.path.join(dirpath, fn)
+            fullpath = pathjoin(dirpath, fn)
             self.assertFalse(fullpath[-1] in '@/')
             info = os.lstat(fullpath)
             if stat.S_ISLNK(info.st_mode):

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2005-12-01 21:31:42 +0000
+++ bzrlib/transport/__init__.py	2005-12-18 04:08:46 +0000
@@ -22,6 +22,7 @@
 from bzrlib.trace import mutter
 import bzrlib.errors as errors
 import errno
+import sys
 
 _protocol_handlers = {
 }
@@ -63,6 +64,12 @@
         if hasattr(e, 'errno'):
             if e.errno in (errno.ENOENT, errno.ENOTDIR):
                 raise errors.NoSuchFile(path, extra=e)
+            # I would rather use errno.EFOO, but there doesn't seem to be
+            # any matching for 267
+            # This is the error when doing a listdir on a file:
+            # WindowsError: [Errno 267] The directory name is invalid
+            if sys.platform == 'win32' and e.errno in (errno.ESRCH, 267):
+                raise errors.NoSuchFile(path, extra=e)
             if e.errno == errno.EEXIST:
                 raise errors.FileExists(path, extra=e)
             if e.errno == errno.EACCES:
@@ -214,30 +221,37 @@
             yield self.get(relpath)
             count += 1
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
         :param f:       File-like or string object.
+        :param mode: The mode for the newly created file, 
+                     None means just use the default
         """
         raise NotImplementedError
 
-    def put_multi(self, files, pb=None):
+    def put_multi(self, files, mode=None, pb=None):
         """Put a set of files or strings into the location.
 
         :param files: A list of tuples of relpath, file object [(path1, file1), (path2, file2),...]
         :param pb:  An optional ProgressBar for indicating percent done.
+        :param mode: The mode for the newly created files
         :return: The number of files copied.
         """
-        return self._iterate_over(files, self.put, pb, 'put', expand=True)
+        def put(path, f):
+            self.put(path, f, mode=mode)
+        return self._iterate_over(files, put, pb, 'put', expand=True)
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         raise NotImplementedError
 
-    def mkdir_multi(self, relpaths, pb=None):
+    def mkdir_multi(self, relpaths, mode=None, pb=None):
         """Create a group of directories"""
-        return self._iterate_over(relpaths, self.mkdir, pb, 'mkdir', expand=False)
+        def mkdir(path):
+            self.mkdir(path, mode=mode)
+        return self._iterate_over(relpaths, mkdir, pb, 'mkdir', expand=False)
 
     def append(self, relpath, f):
         """Append the text in the file-like or string object to 
@@ -267,16 +281,17 @@
         # implementors don't have to implement everything.
         return self._iterate_over(relpaths, self.copy, pb, 'copy', expand=True)
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
+        :param mode: This is the target mode for the newly created files
         TODO: This interface needs to be updated so that the target location
               can be different from the source location.
         """
         # The dummy implementation just does a simple get + put
         def copy_entry(path):
-            other.put(path, self.get(path))
+            other.put(path, self.get(path), mode=mode)
 
         return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
 
@@ -370,6 +385,8 @@
     global _protocol_handlers
     if base is None:
         base = u'.'
+    else:
+        base = unicode(base)
     for proto, klass in _protocol_handlers.iteritems():
         if proto is not None and base.startswith(proto):
             return klass(base)

=== modified file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	2005-12-01 21:31:42 +0000
+++ bzrlib/transport/ftp.py	2005-12-18 04:08:46 +0000
@@ -179,11 +179,13 @@
         except ftplib.error_perm, e:
             raise NoSuchFile(self.abspath(relpath), extra=extra)
 
-    def put(self, relpath, fp):
+    def put(self, relpath, fp, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
         :param f:       File-like or string object.
+        TODO: jam 20051215 This should be an atomic put, not overwritting files in place
+        TODO: jam 20051215 ftp as a protocol seems to support chmod, but ftplib does not
         """
         if not hasattr(fp, 'read'):
             fp = StringIO(fp)
@@ -194,7 +196,7 @@
         except ftplib.error_perm, e:
             raise TransportError(orig_error=e)
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         try:
             mutter("FTP mkd: %s" % self._abspath(relpath))

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-12-03 18:02:21 +0000
+++ bzrlib/transport/http.py	2005-12-18 04:08:46 +0000
@@ -196,7 +196,7 @@
                              % (self.abspath(relpath), str(e)),
                              orig_error=e)
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
@@ -204,7 +204,7 @@
         """
         raise TransportNotPossible('http PUT not supported')
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         raise TransportNotPossible('http does not support mkdir()')
 
@@ -218,7 +218,7 @@
         """Copy the item at rel_from to the location at rel_to"""
         raise TransportNotPossible('http does not support copy()')
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
@@ -232,7 +232,7 @@
         if isinstance(other, HttpTransport):
             raise TransportNotPossible('http cannot be the target of copy_to()')
         else:
-            return super(HttpTransport, self).copy_to(relpaths, other, pb=pb)
+            return super(HttpTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
 
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-12-01 21:31:42 +0000
+++ bzrlib/transport/local.py	2005-12-18 04:08:46 +0000
@@ -26,7 +26,7 @@
 
 from bzrlib.trace import mutter
 from bzrlib.transport import Transport
-from bzrlib.osutils import abspath
+from bzrlib.osutils import abspath, realpath, normpath, pathjoin, rename
 
 
 class LocalTransport(Transport):
@@ -38,8 +38,7 @@
             base = base[7:]
         # realpath is incompatible with symlinks. When we traverse
         # up we might be able to normpath stuff. RBC 20051003
-        super(LocalTransport, self).__init__(
-            os.path.normpath(abspath(base)))
+        super(LocalTransport, self).__init__(normpath(abspath(base)))
 
     def should_cache(self):
         return False
@@ -59,7 +58,7 @@
         This can be supplied with a string or a list
         """
         assert isinstance(relpath, basestring), (type(relpath), relpath)
-        return os.path.join(self.base, urllib.unquote(relpath))
+        return pathjoin(self.base, urllib.unquote(relpath))
 
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path.
@@ -83,7 +82,7 @@
         except (IOError, OSError),e:
             self._translate_error(e, path)
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
@@ -94,7 +93,7 @@
         path = relpath
         try:
             path = self.abspath(relpath)
-            fp = AtomicFile(path, 'wb')
+            fp = AtomicFile(path, 'wb', new_mode=mode)
         except (IOError, OSError),e:
             self._translate_error(e, path)
         try:
@@ -115,12 +114,14 @@
             else:
                 yield relpath
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         path = relpath
         try:
             path = self.abspath(relpath)
             os.mkdir(path)
+            if mode is not None:
+                os.chmod(path, mode)
         except (IOError, OSError),e:
             self._translate_error(e, path)
 
@@ -148,7 +149,7 @@
         path_to = self.abspath(rel_to)
 
         try:
-            os.rename(path_from, path_to)
+            rename(path_from, path_to)
         except (IOError, OSError),e:
             # TODO: What about path_to?
             self._translate_error(e, path_from)
@@ -163,7 +164,7 @@
             # TODO: What about path_to?
             self._translate_error(e, path)
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
@@ -179,13 +180,17 @@
             for path in relpaths:
                 self._update_pb(pb, 'copy-to', count, total)
                 try:
-                    shutil.copy(self.abspath(path), other.abspath(path))
+                    mypath = self.abspath(path)
+                    otherpath = other.abspath(path)
+                    shutil.copy(mypath, otherpath)
+                    if mode is not None:
+                        os.chmod(otherpath, mode)
                 except (IOError, OSError),e:
                     self._translate_error(e, path)
                 count += 1
             return count
         else:
-            return super(LocalTransport, self).copy_to(relpaths, other, pb=pb)
+            return super(LocalTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
 
     def listable(self):
         """See Transport.listable."""

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2005-12-01 21:31:42 +0000
+++ bzrlib/transport/memory.py	2005-12-18 04:08:46 +0000
@@ -67,12 +67,12 @@
             raise NoSuchFile(relpath)
         return StringIO(self._files[relpath])
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """See Transport.put()."""
         self._check_parent(relpath)
         self._files[relpath] = f.read()
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """See Transport.mkdir()."""
         self._check_parent(relpath)
         if relpath in self._dirs:

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2005-12-02 17:14:10 +0000
+++ bzrlib/transport/sftp.py	2006-01-02 22:37:32 +0000
@@ -33,9 +33,10 @@
                            TransportNotPossible, NoSuchFile, PathNotChild,
                            TransportError,
                            LockError)
-from bzrlib.config import config_dir
+from bzrlib.config import config_dir, ensure_config_dir_exists
 from bzrlib.trace import mutter, warning, error
 from bzrlib.transport import Transport, register_transport
+from bzrlib.osutils import pathjoin, fancy_rename
 import bzrlib.ui
 
 try:
@@ -126,6 +127,13 @@
     def send(self, data):
         return os.write(self.proc.stdin.fileno(), data)
 
+    def recv_ready(self):
+        # TODO: jam 20051215 this function is necessary to support the
+        # pipelined() function. In reality, it probably should use
+        # poll() or select() to actually return if there is data
+        # available, otherwise we probably don't get any benefit
+        return True
+
     def recv(self, count):
         return os.read(self.proc.stdout.fileno(), count)
 
@@ -154,7 +162,7 @@
         SYSTEM_HOSTKEYS = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
     except Exception, e:
         mutter('failed to load system host keys: ' + str(e))
-    bzr_hostkey_path = os.path.join(config_dir(), 'ssh_host_keys')
+    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
     try:
         BZR_HOSTKEYS = paramiko.util.load_host_keys(bzr_hostkey_path)
     except Exception, e:
@@ -166,9 +174,9 @@
     Save "discovered" host keys in $(config)/ssh_host_keys/.
     """
     global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
-    bzr_hostkey_path = os.path.join(config_dir(), 'ssh_host_keys')
-    if not os.path.isdir(config_dir()):
-        os.mkdir(config_dir())
+    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
+    ensure_config_dir_exists()
+
     try:
         f = open(bzr_hostkey_path, 'w')
         f.write('# SSH host keys collected by bzr\n')
@@ -191,7 +199,8 @@
         self.lock_path = path + '.write-lock'
         self.transport = transport
         try:
-            self.lock_file = transport._sftp_open_exclusive(self.lock_path)
+            abspath = transport._abspath(self.lock_path)
+            self.lock_file = transport._sftp_open_exclusive(abspath)
         except FileExists:
             raise LockError('File %r already locked' % (self.path,))
 
@@ -316,7 +325,7 @@
         """
         try:
             path = self._abspath(relpath)
-            f = self._sftp.file(path)
+            f = self._sftp.file(path, mode='rb')
             if self._do_prefetch and hasattr(f, 'prefetch'):
                 f.prefetch()
             return f
@@ -342,57 +351,51 @@
             f.prefetch()
         return f
 
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """
         Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
         :param f:       File-like or string object.
+        :param mode: The final mode for the file
         """
         final_path = self._abspath(relpath)
-        tmp_relpath = '%s.tmp.%.9f.%d.%d' % (relpath, time.time(),
+        self._put(final_path, f, mode=mode)
+
+    def _put(self, abspath, f, mode=None):
+        """Helper function so both put() and copy_abspaths can reuse the code"""
+        tmp_abspath = '%s.tmp.%.9f.%d.%d' % (abspath, time.time(),
                         os.getpid(), random.randint(0,0x7FFFFFFF))
-        tmp_abspath = self._abspath(tmp_relpath)
-        fout = self._sftp_open_exclusive(tmp_relpath)
-
+        fout = self._sftp_open_exclusive(tmp_abspath, mode=mode)
+        closed = False
         try:
             try:
+                fout.set_pipelined(True)
                 self._pump(f, fout)
-            except (paramiko.SSHException, IOError), e:
-                self._translate_io_exception(e, relpath, ': unable to write')
+            except (IOError, paramiko.SSHException), e:
+                self._translate_io_exception(e, tmp_abspath)
+            if mode is not None:
+                self._sftp.chmod(tmp_abspath, mode)
+            fout.close()
+            closed = True
+            self._rename(tmp_abspath, abspath)
         except Exception, e:
             # If we fail, try to clean up the temporary file
             # before we throw the exception
             # but don't let another exception mess things up
+            # Write out the traceback, because otherwise
+            # the catch and throw destroys it
+            import traceback
+            mutter(traceback.format_exc())
             try:
-                fout.close()
+                if not closed:
+                    fout.close()
                 self._sftp.remove(tmp_abspath)
             except:
-                pass
-            raise e
-        else:
-            # sftp rename doesn't allow overwriting, so play tricks:
-            tmp_safety = 'bzr.tmp.%.9f.%d.%d' % (time.time(), os.getpid(), random.randint(0, 0x7FFFFFFF))
-            tmp_safety = self._abspath(tmp_safety)
-            try:
-                self._sftp.rename(final_path, tmp_safety)
-                file_existed = True
-            except:
-                file_existed = False
-            success = False
-            try:
-                try:
-                    self._sftp.rename(tmp_abspath, final_path)
-                except (paramiko.SSHException, IOError), e:
-                    self._translate_io_exception(e, relpath, ': unable to rename')
-                else:
-                    success = True
-            finally:
-                if file_existed:
-                    if success:
-                        self._sftp.unlink(tmp_safety)
-                    else:
-                        self._sftp.rename(tmp_safety, final_path)
+                # raise the saved except
+                raise e
+            # raise the original with its traceback if we can.
+            raise
 
     def iter_files_recursive(self):
         """Walk the relative paths of all files in this transport."""
@@ -406,13 +409,20 @@
             else:
                 yield relpath
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         try:
             path = self._abspath(relpath)
+            # In the paramiko documentation, it says that passing a mode flag 
+            # will filtered against the server umask.
+            # StubSFTPServer does not do this, which would be nice, because it is
+            # what we really want :)
+            # However, real servers do use umask, so we really should do it that way
             self._sftp.mkdir(path)
+            if mode is not None:
+                self._sftp.chmod(path, mode=mode)
         except (paramiko.SSHException, IOError), e:
-            self._translate_io_exception(e, relpath, ': unable to mkdir',
+            self._translate_io_exception(e, path, ': unable to mkdir',
                 failure_exc=FileExists)
 
     def _translate_io_exception(self, e, path, more_info='', failure_exc=NoSuchFile):
@@ -439,6 +449,9 @@
             # strange but true, for the paramiko server.
             if (e.args == ('Failure',)):
                 raise failure_exc(path, str(e) + more_info)
+            mutter('Raising exception with args %s', e.args)
+        if hasattr(e, 'errno'):
+            mutter('Raising exception with errno %s', e.errno)
         raise e
 
     def append(self, relpath, f):
@@ -459,7 +472,7 @@
         path_to = self._abspath(rel_to)
         self._copy_abspaths(path_from, path_to)
 
-    def _copy_abspaths(self, path_from, path_to):
+    def _copy_abspaths(self, path_from, path_to, mode=None):
         """Copy files given an absolute path
 
         :param path_from: Path on remote server to read
@@ -474,18 +487,13 @@
         try:
             fin = self._sftp.file(path_from, 'rb')
             try:
-                fout = self._sftp.file(path_to, 'wb')
-                try:
-                    fout.set_pipelined(True)
-                    self._pump(fin, fout)
-                finally:
-                    fout.close()
+                self._put(path_to, fin, mode=mode)
             finally:
                 fin.close()
         except (IOError, paramiko.SSHException), e:
             self._translate_io_exception(e, path_from, ': unable copy to: %r' % path_to)
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
@@ -500,26 +508,29 @@
                 path_from = self._abspath(relpath)
                 path_to = other._abspath(relpath)
                 self._update_pb(pb, 'copy-to', count, total)
-                self._copy_abspaths(path_from, path_to)
+                self._copy_abspaths(path_from, path_to, mode=mode)
                 count += 1
             return count
         else:
-            return super(SFTPTransport, self).copy_to(relpaths, other, pb=pb)
-
-        # The dummy implementation just does a simple get + put
-        def copy_entry(path):
-            other.put(path, self.get(path))
-
-        return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
+            return super(SFTPTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
+
+    def _rename(self, abs_from, abs_to):
+        """Do a fancy rename on the remote server.
+        
+        Using the implementation provided by osutils.
+        """
+        try:
+            fancy_rename(abs_from, abs_to,
+                    rename_func=self._sftp.rename,
+                    unlink_func=self._sftp.remove)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, abs_from, ': unable to rename to %r' % (abs_to))
 
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""
         path_from = self._abspath(rel_from)
         path_to = self._abspath(rel_to)
-        try:
-            self._sftp.rename(path_from, path_to)
-        except (IOError, paramiko.SSHException), e:
-            self._translate_io_exception(e, path_from, ': unable to move to: %r' % path_to)
+        self._rename(path_from, path_to)
 
     def delete(self, relpath):
         """Delete the item at relpath"""
@@ -691,7 +702,7 @@
             save_host_keys()
         if server_key != our_server_key:
             filename1 = os.path.expanduser('~/.ssh/known_hosts')
-            filename2 = os.path.join(config_dir(), 'ssh_host_keys')
+            filename2 = pathjoin(config_dir(), 'ssh_host_keys')
             raise TransportError('Host keys for %s do not match!  %s != %s' % \
                 (self._host, our_server_key_hex, server_key_hex),
                 ['Try editing %s or %s' % (filename1, filename2)])
@@ -776,7 +787,7 @@
             pass
         return False
 
-    def _sftp_open_exclusive(self, relpath):
+    def _sftp_open_exclusive(self, abspath, mode=None):
         """Open a remote path exclusively.
 
         SFTP supports O_EXCL (SFTP_FLAG_EXCL), which fails if
@@ -787,19 +798,22 @@
         WARNING: This breaks the SFTPClient abstraction, so it
         could easily break against an updated version of paramiko.
 
-        :param relpath: The relative path, where the file should be opened
+        :param abspath: The remote absolute path where the file should be opened
+        :param mode: The mode permissions bits for the new file
         """
-        path = self._sftp._adjust_cwd(self._abspath(relpath))
+        path = self._sftp._adjust_cwd(abspath)
         attr = SFTPAttributes()
-        mode = (SFTP_FLAG_WRITE | SFTP_FLAG_CREATE 
+        if mode is not None:
+            attr.st_mode = mode
+        omode = (SFTP_FLAG_WRITE | SFTP_FLAG_CREATE 
                 | SFTP_FLAG_TRUNC | SFTP_FLAG_EXCL)
         try:
-            t, msg = self._sftp._request(CMD_OPEN, path, mode, attr)
+            t, msg = self._sftp._request(CMD_OPEN, path, omode, attr)
             if t != CMD_HANDLE:
                 raise TransportError('Expected an SFTP handle')
             handle = msg.get_string()
-            return SFTPFile(self._sftp, handle, 'w', -1)
+            return SFTPFile(self._sftp, handle, 'wb', -1)
         except (paramiko.SSHException, IOError), e:
-            self._translate_io_exception(e, relpath, ': unable to open',
+            self._translate_io_exception(e, abspath, ': unable to open',
                 failure_exc=FileExists)
 

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-11-23 13:40:57 +0000
+++ bzrlib/tree.py	2005-12-19 20:31:01 +0000
@@ -116,29 +116,30 @@
            or at least passing a description to the constructor.
     """
     
-    def __init__(self, weave_store, inv, revision_id):
-        self._weave_store = weave_store
+    def __init__(self, branch, inv, revision_id):
+        self._branch = branch
+        self._weave_store = branch.weave_store
         self._inventory = inv
         self._revision_id = revision_id
 
     def get_weave(self, file_id):
-        # FIXME: RevisionTree should be given a branch
-        # not a store, or the store should know the branch.
         import bzrlib.transactions as transactions
         return self._weave_store.get_weave(file_id,
-            transactions.PassThroughTransaction())
+                self._branch.get_transaction())
 
+    def get_weave_prelude(self, file_id):
+        import bzrlib.transactions as transactions
+        return self._weave_store.get_weave_prelude(file_id,
+                self._branch.get_transaction())
 
     def get_file_lines(self, file_id):
         ie = self._inventory[file_id]
         weave = self.get_weave(file_id)
         return weave.get(ie.revision)
-        
 
     def get_file_text(self, file_id):
         return ''.join(self.get_file_lines(file_id))
 
-
     def get_file(self, file_id):
         return StringIO(self.get_file_text(file_id))
 

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-11-23 06:25:50 +0000
+++ bzrlib/upgrade.py	2005-12-01 18:16:47 +0000
@@ -81,7 +81,7 @@
 from bzrlib.xml4 import serializer_v4
 from bzrlib.xml5 import serializer_v5
 from bzrlib.trace import mutter, note, warning
-from bzrlib.osutils import sha_strings, sha_string
+from bzrlib.osutils import sha_strings, sha_string, pathjoin, abspath
 
 
 class Convert(object):
@@ -108,7 +108,7 @@
             note('starting upgrade from format 5 to 6')
             self._convert_to_prefixed()
             self._open_branch()
-        cache = hashcache.HashCache(os.path.abspath(self.base))
+        cache = hashcache.HashCache(abspath(self.base))
         cache.clear()
         cache.write()
         note("finished")
@@ -118,17 +118,17 @@
         from bzrlib.store import hash_prefix
         for store_name in ["weaves", "revision-store"]:
             note("adding prefixes to %s" % store_name) 
-            store_dir = os.path.join(self.base, ".bzr", store_name)
+            store_dir = pathjoin(self.base, ".bzr", store_name)
             for filename in os.listdir(store_dir):
                 if filename.endswith(".weave") or filename.endswith(".gz"):
                     file_id = os.path.splitext(filename)[0]
                 else:
                     file_id = filename
-                prefix_dir = os.path.join(store_dir, hash_prefix(file_id))
+                prefix_dir = pathjoin(store_dir, hash_prefix(file_id))
                 if not os.path.isdir(prefix_dir):
                     os.mkdir(prefix_dir)
-                os.rename(os.path.join(store_dir, filename),
-                          os.path.join(prefix_dir, filename))
+                os.rename(pathjoin(store_dir, filename),
+                          pathjoin(prefix_dir, filename))
         self._set_new_format(BZR_BRANCH_FORMAT_6)
 
 

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2005-12-13 03:54:33 +0000
+++ bzrlib/weave.py	2005-12-20 02:45:27 +0000
@@ -67,12 +67,14 @@
 # the possible relationships.
 
 
+import os
 import sha
 from difflib import SequenceMatcher
 
 from bzrlib.trace import mutter
-from bzrlib.errors import WeaveError, WeaveFormatError, WeaveParentMismatch, \
-        WeaveRevisionNotPresent, WeaveRevisionAlreadyPresent
+from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
+        WeaveRevisionNotPresent, WeaveRevisionAlreadyPresent)
+import bzrlib.errors as errors
 from bzrlib.tsort import topo_sort
 
 
@@ -483,7 +485,12 @@
                 yield lineno, istack[-1], dset, l
             lineno += 1
 
-
+        if istack:
+            raise WeaveFormatError("unclosed insertion blocks "
+                    "at end of weave: %s" % istack)
+        if dset:
+            raise WeaveFormatError("unclosed deletion blocks at end of weave: %s"
+                                   % dset)
 
     def _extract(self, versions):
         """Yield annotation of lines in included set.
@@ -548,8 +555,24 @@
     def get_iter(self, name_or_index):
         """Yield lines for the specified version."""
         incls = [self.maybe_lookup(name_or_index)]
+        if len(incls) == 1:
+            index = incls[0]
+            cur_sha = sha.new()
+        else:
+            # We don't have sha1 sums for multiple entries
+            cur_sha = None
         for origin, lineno, line in self._extract(incls):
+            if cur_sha:
+                cur_sha.update(line)
             yield line
+        if cur_sha:
+            expected_sha1 = self._sha1s[index]
+            measured_sha1 = cur_sha.hexdigest() 
+            if measured_sha1 != expected_sha1:
+                raise errors.WeaveInvalidChecksum(
+                        'file %s, revision %s, expected: %s, measured %s' 
+                        % (self._weave_name, self._names[index],
+                           expected_sha1, measured_sha1))
 
 
     def get_text(self, name_or_index):
@@ -564,6 +587,13 @@
     get = get_lines
 
 
+    def get_sha1(self, name):
+        """Get the stored sha1 sum for the given revision.
+        
+        :param name: The name of the version to lookup
+        """
+        return self._sha1s[self.lookup(name)]
+
     def mash_iter(self, included):
         """Return composed version of multiple included versions."""
         included = map(self.maybe_lookup, included)
@@ -589,7 +619,6 @@
     def __len__(self):
         return self.numversions()
 
-
     def check(self, progress_bar=None):
         # check no circular inclusions
         for version in range(self.numversions()):
@@ -600,27 +629,53 @@
                     raise WeaveFormatError("invalid included version %d for index %d"
                                            % (inclusions[-1], version))
 
-        # try extracting all versions; this is a bit slow and parallel
-        # extraction could be used
+        # try extracting all versions; parallel extraction is used
         nv = self.numversions()
+        sha1s = [sha.new() for i in range(nv)]
+        texts = [[] for i in range(nv)]
+        inclusions = []
+        for i in range(nv):
+            # For creating the ancestry, IntSet is much faster (3.7s vs 0.17s)
+            # The problem is that set membership is much more expensive
+            new_inc = set([i])
+            for p in self._parents[i]:
+                new_inc.update(inclusions[p])
+
+            #assert set(new_inc) == self.inclusions([i]), 'failed %s != %s' % (new_inc, self.inclusions([i]))
+            inclusions.append(new_inc)
+
+        nlines = len(self._weave)
+
+        update_text = 'checking weave'
+        if self._weave_name:
+            short_name = os.path.basename(self._weave_name)
+            update_text = 'checking %s' % (short_name,)
+            update_text = update_text[:25]
+
+        for lineno, insert, deleteset, line in self._walk():
+            if progress_bar:
+                progress_bar.update(update_text, lineno, nlines)
+
+            for j, j_inc in enumerate(inclusions):
+                # The active inclusion must be an ancestor,
+                # and no ancestors must have deleted this line,
+                # because we don't support resurrection.
+                if (insert in j_inc) and not (deleteset & j_inc):
+                    sha1s[j].update(line)
+
         for version in range(nv):
-            if progress_bar:
-                progress_bar.update('checking text', version, nv)
-            s = sha.new()
-            for l in self.get_iter(version):
-                s.update(l)
-            hd = s.hexdigest()
+            hd = sha1s[version].hexdigest()
             expected = self._sha1s[version]
             if hd != expected:
-                raise WeaveError("mismatched sha1 for version %d; "
-                                 "got %s, expected %s"
-                                 % (version, hd, expected))
+                raise errors.WeaveInvalidChecksum(
+                        "mismatched sha1 for version %s: "
+                        "got %s, expected %s"
+                        % (self._names[version], hd, expected))
 
         # TODO: check insertions are properly nested, that there are
         # no lines outside of insertion blocks, that deletions are
         # properly paired, etc.
 
-
     def _delta(self, included, lines):
         """Return changes from basis to new revision.
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-12-12 07:20:02 +0000
+++ bzrlib/workingtree.py	2006-01-02 22:54:45 +0000
@@ -63,9 +63,13 @@
                             compact_date,
                             file_kind,
                             isdir,
+                            getcwd,
+                            pathjoin,
                             pumpfile,
                             splitpath,
                             rand_bytes,
+                            abspath,
+                            normpath,
                             realpath,
                             relpath,
                             rename)
@@ -193,8 +197,6 @@
         self.branch = branch
         self.basedir = realpath(basedir)
 
-        self._set_inventory(self.read_working_inventory())
-
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
         # two possible ways offer themselves : in self._unlock, write the cache
@@ -209,6 +211,8 @@
             mutter("write hc")
             hc.write()
 
+        self._set_inventory(self.read_working_inventory())
+
     def _set_inventory(self, inv):
         self._inventory = inv
         self.path2id = self._inventory.path2id
@@ -225,12 +229,12 @@
         If there is one, it is returned, along with the unused portion of path.
         """
         if path is None:
-            path = os.getcwdu()
+            path = getcwd()
         else:
             # sanity check.
             if path.find('://') != -1:
                 raise NotBranchError(path=path)
-        path = os.path.abspath(path)
+        path = abspath(path)
         tail = u''
         while True:
             try:
@@ -238,12 +242,12 @@
             except NotBranchError:
                 pass
             if tail:
-                tail = os.path.join(os.path.basename(path), tail)
+                tail = pathjoin(os.path.basename(path), tail)
             else:
                 tail = os.path.basename(path)
+            lastpath = path
             path = os.path.dirname(path)
-            # FIXME: top in windows is indicated how ???
-            if path == os.path.sep:
+            if lastpath == path:
                 # reached the root, whatever that may be
                 raise NotBranchError(path=path)
 
@@ -263,11 +267,11 @@
                                getattr(self, 'basedir', None))
 
     def abspath(self, filename):
-        return os.path.join(self.basedir, filename)
+        return pathjoin(self.basedir, filename)
 
-    def relpath(self, abspath):
+    def relpath(self, abs):
         """Return the local path portion from a given absolute path."""
-        return relpath(self.basedir, abspath)
+        return relpath(self.basedir, abs)
 
     def has_filename(self, filename):
         return bzrlib.osutils.lexists(self.abspath(filename))
@@ -314,6 +318,7 @@
     def get_file_size(self, file_id):
         return os.path.getsize(self.id2abspath(file_id))
 
+    @needs_read_lock
     def get_file_sha1(self, file_id):
         path = self._inventory.id2path(file_id)
         return self._hashcache.get_sha1(path)
@@ -372,7 +377,7 @@
             if len(fp) == 0:
                 raise BzrError("cannot add top-level %r" % f)
 
-            fullpath = os.path.normpath(self.abspath(f))
+            fullpath = normpath(self.abspath(f))
 
             try:
                 kind = file_kind(fullpath)
@@ -909,6 +914,8 @@
         between multiple working trees, i.e. via shared storage, then we 
         would probably want to lock both the local tree, and the branch.
         """
+        if self._hashcache.needs_write:
+            self._hashcache.write()
         return self.branch.unlock()
 
     @needs_write_lock

